//==============================================================================
// aq_class_commonSDPheader.c
//
// 
// 18 May 2001 Version 1.0
//
// Purpose:
//     This class defines portion of SDP Header Standard Sructure common to both
//     ephemeris and attitude files.
//
// Note:
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
//
//==============================================================================

#include  <string.h>
#include  <stdio.h>

#include  "constlib_TimeConstants.h"
#include  "timelib_class_datetime.h"
#include  "aq_class_commonSDPheader.h"
#include  "aq_proc_formatconvert_x.h"

static const double Jan1993 = time_const_01Jan1993_SDP;



//------------------------------------------------------------------------------
// UTC to TAI
//
//	See Theoretical Basis of the SDP Toolkit Geolocation Package for the
//	ECS Project, 445-TP-002-002, May 1995, Section 4.6.3
//
//	2394566.0 JD is 0 Jan 1844;
//      32.18 sec is TDT-UT1 1 Jan 1958.
//------------------------------------------------------------------------------

static double TAItoUTC( double tai )
{
    double delta = (0.537L/2.0) * pow( (tai - 2394566.0)/365.25, 2) / 100.0
	         - 32.18;

    return tai - delta/86400;
}


//------------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------------

commonSDPheader::commonSDPheader() : storebase(140)
{
    ptr = DataPtr();
}

    
//------------------------------------------------------------------------------
// SpacecraftID
//		beg = 00  end = 23  cnt = 24 	ascii spacraft id
//
//------------------------------------------------------------------------------

commonSDPheader& commonSDPheader::SpacecraftID( const string& value )
{
    value.copy( &ptr[0], 24 );
    return *this;
} 

   
string commonSDPheader::SpacecraftID() const
{
    return string( ptr, 24 );
} 


//------------------------------------------------------------------------------
// Source
//		beg = 72  end = 103  cnt = 32 	ascii data source
//
//------------------------------------------------------------------------------

commonSDPheader& commonSDPheader::Source ( const string& value )
{ 
    value.copy( &ptr[72], 32 );
    return *this;
}


string commonSDPheader::Source()  const
{
    return string( ptr+72, 32 );
}


//page
//------------------------------------------------------------------------------
// Version
//		beg = 104  end = 111  cnt = 8 	ascii data source
//
//------------------------------------------------------------------------------


commonSDPheader& commonSDPheader::Version( const string& value )
{
    value.copy( &ptr[104], 8 );
    return *this;
}


string commonSDPheader::Version() const
{
    return string( ptr+104, 8 );
}



//------------------------------------------------------------------------------
// TimeRange
//		beg = 112  end = 119   cnt =  8  double start time TAI93
//		beg = 120  end = 127   cnt =  8  double stop  time TAI93
//		beg = 24   end =  71   cnt = 48  ascii  time range
//------------------------------------------------------------------------------

     
commonSDPheader& commonSDPheader::TimeRange( double _startTime, double _endTime)
{
    FormatConvert_float8( (_startTime - Jan1993) * 86400.0, ptr+112);
    FormatConvert_float8( (_endTime   - Jan1993) * 86400.0, ptr+120);

    size_t year;
    size_t month;
    size_t day;
    size_t hour;
    size_t min;
    double sec;
    
    date_time timeJD( TAItoUTC(_startTime)  );
    timeJD.GetGregorianDate( year, month, day, hour, min, sec );

    sprintf
    	( &ptr[24], "%4d-%02d-%02dT%02d:%02d:%02dZ to ", 
      	  year, month, day, hour, min, int(sec) );

    timeJD.SetJulianDay( TAItoUTC(_endTime) );
    timeJD.GetGregorianDate( year, month, day, hour, min, sec );

    sprintf
    	( &ptr[49], "%4d-%02d-%02dT%02d:%02d:%02dZ ", 
	  year, month, day, hour, min, int(sec) );

    return *this;
}


string commonSDPheader::TimeRange() const
{
    return string( ptr+24, 48 );
}


double commonSDPheader::StartTime()  const
{
    return (FormatConvert_float8( ptr+112 ) / 86400.0) + Jan1993;
}


double commonSDPheader::StopTime()  const
{
    return (FormatConvert_float8( ptr+120 ) / 86400.0) + Jan1993;
}

//page
//------------------------------------------------------------------------------
// Interval
//		beg = 128 end = 131  cnt =  4  float interval between records
//------------------------------------------------------------------------------

commonSDPheader& commonSDPheader::Interval( float interval )
{
    FormatConvert_float4( interval, ptr+128 );
    return *this;
}


float commonSDPheader::Interval() const
{
    return FormatConvert_float4( ptr+128 );
}


//------------------------------------------------------------------------------
// NoRecords
//		beg = 136 end = 139   cnt =  4  uint interval between records
//------------------------------------------------------------------------------

commonSDPheader& commonSDPheader::NoRecords( uint rcds )
{
    FormatConvert_long4( rcds, ptr+136 );
    return *this;
}


uint commonSDPheader::NoRecords() const
{
    return FormatConvert_long4( ptr+136 );
}


//------------------------------------------------------------------------------
// NoURefs
//		beg = 132 end = 135   cnt =  4  uint No. of Universal References
//------------------------------------------------------------------------------
    
uint   commonSDPheader::NoURefs()   const
{
    return FormatConvert_long4( ptr+132 );
}







