//==============================================================================
// aq_class_ephemerismetadata.c
//
// 
// 17 May 2001 Version 1.0
// 28 May 2002         1.1 - add <iostream>
// 27 Jun 2002	       1.1 - set storebase to 32 bytes
//
// Purpose:
// 	This class defines an ephemeris output file in SDP format.
//
// Note:
//	Headers are written when file is closed, so headers can be updated any
//	time before closure.  Headers passed to file constructor must not be
//	destroyed before file closure.
//
// 	General format:
//	SDP Header
//	ephemeris subheader
//	ephemeris records
//	metadata  records
//
//	See SDP Toolkit 333-CD-600-001 Appendix L, Ephemeris and Attitude File
//	Formats.
//
//==============================================================================

#include <string.h>
#include <iostream>

#include "aq_class_ephemerismetadata.h"
#include "aq_typdef_header.h"
#include "aq_proc_formatconvert_x.h"		// machine-dependent formats

static const double Jan1993 = 2448988.5003125;	// 01 Jan 1993 TAI Julian Date


//------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------

ephemerismetadata::ephemerismetadata() : storebase(32)
{  }


//page
//------------------------------------------------------------------------
// OrbitNo
//              beg = 00  end = 03  cnt = 04    uint orbit number
//
//------------------------------------------------------------------------

ephemerismetadata&  ephemerismetadata::OrbitNo( uint orbitNo )
{
    FormatConvert_long4( orbitNo, DataPtr() );
    return *this;
}

    
uint  ephemerismetadata::OrbitNo() const
{
    return FormatConvert_long4( DataPtr() );
}


//------------------------------------------------------------------------
// AscendingTime-
//      beg = 08  end = 15  cnt = 08    uint orbit number
//
//------------------------------------------------------------------------

ephemerismetadata& ephemerismetadata::AscendingTime(  double time )
{

	if(time < 0)
		 FormatConvert_float8( -999.99999999, DataPtr() + 8);
	else
    	FormatConvert_float8( (time-Jan1993)*86400.0, DataPtr()+8 );
    return *this;
}

    
double ephemerismetadata::AscendingTime()  const
{
	double ascTime = FormatConvert_float8(DataPtr()+8);
	
	if(ascTime < 0)
		return FormatConvert_float8( DataPtr() + 8);
	else
    	return FormatConvert_float8( DataPtr()+8 )/86400.0 + Jan1993;
}

    
double ephemerismetadata::AscendingTimeTAI()  const  // diagnostic
{
    return FormatConvert_float8( DataPtr()+8 );
}


//page
//------------------------------------------------------------------------
// DescendingTime-
//      beg = 16  end = 23  cnt = 08     
//
//------------------------------------------------------------------------

ephemerismetadata& ephemerismetadata::DescendingTime(  double time )
{
    FormatConvert_float8( (time-Jan1993)*86400.0, DataPtr()+16 );
    return *this;
}


double ephemerismetadata::DescendingTime() const
{
    return FormatConvert_float8( DataPtr()+16 )/86400.0 + Jan1993;
}


//------------------------------------------------------------------------
// DescendingLongitude-
//              beg = 24  end = 31  cnt = 08    
//
//------------------------------------------------------------------------

ephemerismetadata& ephemerismetadata::DescendingLongitude(  double lng )
{
    FormatConvert_float8( lng, DataPtr()+24 );
    return *this;
}


double ephemerismetadata::DescendingLongitude() const
{
    return FormatConvert_float8( DataPtr()+24 ) ;
}


//------------------------------------------------------------------------
// print
//------------------------------------------------------------------------


ostream& operator<<(ostream& os, const ephemerismetadata& ephemdata )
{
    
    os<<"\nOrbit number   = "<<ephemdata.OrbitNo()
      <<"\nAsc. Node time = "<<ephemdata.AscendingTime()
      <<"\nDesc Node Time = "<<ephemdata.DescendingTime()
      <<"\nDesc Longitude = "<<ephemdata.DescendingLongitude()
      <<endl;

    return os;
}                                 









