//==============================================================================
// aq_class_satpasses.c
//
// 
// 18 May 2001 Version 1.0
// 28 Jun 2002         1.1 - fix beg and end revno equation error
//			   - fix NORAD TLE to Julian Day offset error
// 03 Jul 2002             - fix Asc, desc node time equations
// 08 Jul 2002		   - fix desc. node terrestrial longitude eq. 
//			   - add ndot to right ascension for desc.node long.
//
// 01 Nov 2002	       1.2 - constrain asc. node longitude to +/- pi radians.
// 04 Nov 2002		   - kepler elements M constrain to pos principle ang.
//			   - add EpochTimeTAI93 method.
//
// 07 Aug 2003	       1.3 - add asctime, begkepler to print_state
//			   - change (size_t) to (int) cast in ascTime equation
//			   - in SetOrbit()
//
// Object to get orbital parameters from orbitstuff structure and NORAD two 
// line elements
//
// Note:
//	See "Satellite Times", January 1998 for NORAD TLE time and date 
//	description. e.g: epoch of 98001.00000000 = 00:00 UT 1998 Jan 01, i.e.,
//	midnight between Dec 31 and Jan 01. 
//
//==============================================================================

#include <iomanip>
#include <math.h>

#include  "aq_class_satpasses.h"
#include  "aq_proc_printarray.h"
#include  "constlib_MathConstants.h"
#include  "constlib_TimeConstants.h"
#include  "aq_typdef_header.h"
#include  "orbitlib_proc_GreenwichHourAngle.h"
#include  "orbitlib_proc_ConvertRectangularToKepler.h"
#include  "timelib_class_datetime.h"
#include  "aq_proc_printarray.h"
#include  <assert.h>

static const double&  c_2pi = math_const_twoPi;
static const double&  c_pi  = math_const_pi;


//------------------------------------------------------------------------------
// UTC to TAI
//
//	See Theoretical Basis of the SDP Toolkit Geolocation Package for the
//	ECS Project, 445-TP-002-002, May 1995, Section 4.6.3
//
//	2394566.0 JD is 0 Jan 1844;
//      32.18 sec is TDT-UT1 1 Jan 1958.
//------------------------------------------------------------------------------
 
static double UTC_TAI_delta( double utc )
{
    double delta = (0.537L/2.0) * pow( (utc - 2394566.0)/365.25, 2) / 100.0
	         - 32.18;

    return delta/86400;
}


//page
//------------------------------------------------------------------------------
// Constructor
//
//------------------------------------------------------------------------------
    
satpasses::satpasses( const noradelements& noradtle )
{
    assert( (noradtle.Get()).size()  == 13 );

    carray noradel = noradtle.Get();

    epochRev = (size_t)noradel[12] ;	// Revolution number at Epoch
    meanMo   = noradel[0] * 1440.0 ;	// Radians / Day
    rightAsc = noradel[3];		// Right Ascension (rad)

					// Y2K fix
    size_t year =
	(size_t)( (noradel[10] < 50) ? noradel[10]+2000 : noradel[10]+1900 );

					// Julian Epoch
    date_time epochT;
    epochT.SetGregorianYearDaySecond( year, 0, (noradel[11] - 1.0) * 86400.0 );
    epochTime = epochT.GetJulianDay();

    TAIminusUTC = UTC_TAI_delta(epochTime);
    epochTime  += TAIminusUTC;		// convert epochTime to TAI 
					// (approx 35/86400 )

					// perigee + mean anomaly at epoch
    wPlusM_Ep = noradel[4] + noradel[5];

					// ascending node time for epoch rev.
    epochAscTime = epochTime - wPlusM_Ep / meanMo;

    begTime  = 0.;			// pass start time (JD)	
    endTime  = 0.;			// pass stop  time (JD)

    ascTime  = 0.;			// orbit ascending  node time
    dscTime  = 0.;			// orbit descending node time
}


//page
//------------------------------------------------------------------------------
// SetOrbit
//
//------------------------------------------------------------------------------

satpasses& satpasses::SetOrbit( const orbitstuff& orbit )
{
    const gbadpacket &begPkt = orbit.beginpkt;
    
    assert( begPkt.Position().size() == 3 );
    assert( begPkt.Velocity().size() == 3 );

    begTime = begPkt.TimeTag();		// pass start (JD)
    endTime = orbit.endTime;		// pass end   (JD)


					// XYZ position with velocity apppended 
					// in J2000 reference. 
    array posVel = begPkt.Position();
    for( size_t i=0; i<3; i++) posVel.push_back( begPkt.Velocity()[i] );

          				// Convert to keplerian elements;
    begKepler  = Convert_RectangularToKepler( posVel );

					// perigee + mean anomaly at pass start
    wPlusM_Ps = begKepler[4] + begKepler[5];
    if( wPlusM_Ps> c_2pi ) wPlusM_Ps -= c_2pi;

					 
    double RevDelta = (begTime -epochAscTime) * meanMo / c_2pi ;

    ascTime = epochAscTime + (int)RevDelta * c_2pi  / meanMo;
    dscTime = ascTime + c_pi / meanMo;

    return *this;
}


//page
//------------------------------------------------------------------------------
// EpochTimeTAI93
//
//------------------------------------------------------------------------------

double satpasses::EpochTimeTAI93() const
{
    return (epochTime - time_const_01Jan1993_SDP) * 86400;
}


//------------------------------------------------------------------------------
// BegKepler-
//
//	Keplerian Elements at beginning of pass
//------------------------------------------------------------------------------

array satpasses::BegKepler() const
{
    if( begKepler.size() != 6 ) return array(6);	// error

    array  temp = begKepler;
    temp[5] = temp[5] - (wPlusM_Ps - wPlusM_Ep);

    temp[5] = fmod(temp[5], c_2pi);		// reduce to principle angle
    if( temp[5] < 0.0 ) temp[5] += c_2pi;	// positive angle

    return temp;
}


//page
//------------------------------------------------------------------------------
// BegOrbitRev
//
//------------------------------------------------------------------------------

size_t satpasses::BegOrbitRev()  const
{
    if( begTime == 0 ) return 0;			// error

    double RevDelta = (begTime -epochAscTime) * meanMo /c_2pi;
    return epochRev +  (size_t)RevDelta;
}


//------------------------------------------------------------------------------
// EndOrbitRev
//
//------------------------------------------------------------------------------

size_t satpasses::EndOrbitRev()  const
{
    if( endTime == 0 ) return 0; 			// error
 
    double RevDelta = (endTime -epochAscTime) * meanMo /c_2pi;
    return epochRev +  (size_t)RevDelta;
}
   



//------------------------------------------------------------------------------
// BegOrbitTime
//
//------------------------------------------------------------------------------

double satpasses::BegOrbitTime() const
{
    return begTime;
}


//------------------------------------------------------------------------------
// EndOrbitTime
//
//------------------------------------------------------------------------------

double satpasses::EndOrbitTime() const
{
    return endTime;
}


//page
//------------------------------------------------------------------------------
// AscendingNodeTime
//
//------------------------------------------------------------------------------

double satpasses::AscendingNodeTime()  const
{
    return ascTime;
}


//------------------------------------------------------------------------------
// DescendingNodeTime 
//
//------------------------------------------------------------------------------

double satpasses::DescendingNodeTime()  const
{
    return dscTime;
}


//------------------------------------------------------------------------------
// DescendingNodeLongitude
//
//------------------------------------------------------------------------------

double satpasses::DescendingNodeLongitude()  const
{
    if( dscTime == 0 ) return 0.;
					//    0.01728 rad/day ~ ndot
    double ascDelta = (dscTime - epochTime) * 0.01728;

    double tmp = 
    rightAsc + ascDelta - GreenwichHourAngle( dscTime-TAIminusUTC ) + c_pi;

    tmp = fmod(tmp, c_2pi);		// reduce to principle angle
    if( tmp < 0.0 ) tmp += c_2pi;	// positive angle
    if( tmp > c_pi) tmp -= c_2pi;	// convert to east/west longitude

    return tmp;
}


//page
//------------------------------------------------------------------------------
// print_state
//
//------------------------------------------------------------------------------

void satpasses::print_state( ostream& os )
{
    os.setf(ios::fixed | ios::showpos );

    os<<"\nsatpasses state "
      <<"\nbegTime   = "<<begTime
      <<"\nendTime   = "<<endTime
      <<"\nascTime   = "<<ascTime
      <<"\ndscTime   = "<<dscTime
      <<"\nepochRev  = "<<epochRev
      <<"\nepochTime = "<<epochTime
      <<"\nepAsctime = "<<epochAscTime
      <<"\nmeanMO    = "<<meanMo
      <<"\nwPlusM_Ep = "<<wPlusM_Ep
      <<"\nwPlusM_Ps = "<<wPlusM_Ps
      <<"\nrightAsc  = "<<rightAsc
      <<"\nbegKepler = "<<begKepler
      <<endl;
}


//------------------------------------------------------------------------------
// operator<<
//
//------------------------------------------------------------------------------

ostream& operator<<( ostream& os, const satpasses& satp )
{
    os<<'\n'
      <<"Beg Orbit Rev = "<<satp.BegOrbitRev()
      <<"  Time = "<<satp.BegOrbitTime()

      <<'\n'
      <<"End Orbit Rev = "<<satp.EndOrbitRev()
      <<"  Time = "<<satp.EndOrbitTime()

      <<'\n'<<"Ascending  Node Time = "<<satp.AscendingNodeTime()
      <<'\n'<<"Descending Node Time = "<<satp.DescendingNodeTime()
      <<'\n'<<"Descending Longitude = "<<satp.DescendingNodeLongitude()

      <<'\n'
      <<"Keplerian  Elements  =\n    "<<satp.BegKepler()
      <<endl;

    return os;
}











