//=============================================================================
// aq_class_stackbuffer.h
//
//  
// 07 Jan 2002 Version 1.0
//
// Description:
//  	Implement a fixed-size stack of indexable buffers where new objects are 
//	added at bottom and, when stack is full, old objects are removed from top.
//
// 	Indices are manipulated, rather than copying objects so that large 
//	objects are handled more efficiently
//
// Usage:
//
// 	template<class T> class stackbuffer
//	 {
//	 public:
//	    stackbuffer( size_t size ) : buffer( _size = size);
//	    void   clear();
//	    T*     add();
//	    T*     peek( size_t index );
//	    const  T* peek( size_t index ) const;
//	    bool   full()   const;
//	    size_t length() const;		// number of objects in buffer
//	    size_t size()   const;		// max no. of objects allowed
//          virtual ~stackbuffer();
//	    void   print( ostream& os = cout)   // diagnostic
//	};
//
//
//	add() returns a pointer to the next buffer to be loaded and increments
//	the length. The buffer need not actually be loaded.  When length equals
//	stackbuffer size, the buffer is full, and old objects are removed from 
//	the top of stack. add() returns null if buffer size is zero.
//
//	peek(index) returns a pointer to one of the buffers. Range of index is 
//	0 to length-1.  If index is out of range, a null pointer is returned.
//	peek(0) buffer points to oldest object. peek(index) does not change stack.
//	
//=============================================================================

#if !defined(AQ_CLASS_STACKBUFFER)
#define      AQ_CLASS_STACKBUFFER

#include <iostream>
#include <vector>
#include <iomanip>

//page
template<class T> class stackbuffer
{
    vector<T> buffer;		// actual list of objects.
    int      _length;		// objects in buffer.
    size_t   _kindex;		// implement circular buffer.
    int      _lindex;		// last buffer loaded.
    size_t   _size;		// size of buffer.

    T* cvtgetindex( size_t uindex ) const
    {
	if( (int)uindex >= _size ) return 0;

	int _gindex = uindex + _kindex;
	if( _gindex >= _length ) _gindex -= _length;

	return const_cast<T*>(&buffer[ _gindex ]);
    }

public:
    explicit stackbuffer( size_t size ) : buffer( _size = size)
    {
	clear();
    }

    void clear()
    {
	_length = 0;
	_kindex = 0;
	_lindex =-1;
    }


    T*  add()
    {
	if( !_size ) return 0;

	if( full() && ++_kindex == _size ) _kindex = 0;
	if( _length <  _size ) _length++;

	return &buffer[ _lindex = ++_lindex % _size];
    } 


    T*  peek( size_t uindex ) { return cvtgetindex(uindex); }
    const T* peek( size_t uindex) const {return cvtgetindex(uindex) ;}

    bool   full()   const { return _length == _size;}
    size_t length() const { return _length; }
    size_t size()   const { return _size;   }

    virtual ~stackbuffer() { }
};


//page
//-----------------------------------------------------------------------------
// operator<<
//	diagnostic print
//-----------------------------------------------------------------------------

template<class T>
static ostream& operator<<( ostream& os, const stackbuffer<T>& buf )
{
    os<<"full="<<buf.full()<<"  length="<<setw(2)
      <<buf.length()<<"  size="<<setw(2)<<buf.size()<<' '<<endl;

    return os;
}


#endif










