//==============================================================================
// aq_proc_Mil1750_EPFP.c
//
// 
// 17 May 2001 Version 1.0
//
// 17 Nov 2003 Version 1.1- rewrite to correct negative last part of mantissa
//							problem. 
//
// Purpose:
// 	Convert squirrely MIL-STD-1750A Extended Precision Floating Point 
//	value to IEEE double precision number. 
//
//	See "Mil-Std-1750A, Notice 1, Sixteen-Bit Computer Instruction Set
//	Architecture, July 1980", section 4.1.6. 
//
// Usage: 
//	ptr points to 6-byte 1750A Extended Precision Floating Point Number
//
// Method:
//  "m1" is loaded with first 3 bytes of 2's complement mantissa; last (right) 
//  byte is the (unwanted) exponent. First (left) bit is sign bit followed by
//  implied decimal point. 
//  
//  Divide m1 by 256 to shift (positive or negative) m1 right 8 bits. 
//
//  Get 2's complement exponent "exp", and get last two bytes of mantissa, "m2".
//  
//  If mantissa is negative, change to positive by 2's complementing: complement
//  entire mantissa (m1 and m2), and add 1. If m2 is zero, carry the 1 to m1.
//
//  Now entire mantissa is absolute value of original (signed value).
//
//  Multiply m1 by value of m1 lsb (would be 2e-31 but already divided by 256 so 
//  correct value is -23). 
//
//	Multiply m2 by value of m2 lsb = 2e-39.
//
//  add values, scale to exponent, and restore algebraic sign. 
//
//==============================================================================

#include  <math.h>
#include  <string.h>
#include  "aq_proc_formatconvert_x.h"

static const double c_2pM39 = pow(2.0, -39);	// LSB of last  part of mantissa,
static const double c_2pM23 = pow(2.0, -23);	// LSB of first part of mantissa
												// after /256 to remove exponent.

//page
//------------------------------------------------------------------------------
//  MiL1750_EPFP
//		- 6-byte float
//------------------------------------------------------------------------------

double MiL1750_EPFP( char* ptr )
{    	
	long m1 = FormatConvert_long4( ptr );				// first part of mantissa 
	m1 /= 256;											// remove exponent
		
	int exp = FormatConvert_schar(ptr+3);				// exponent   

	unsigned short m2 = FormatConvert_ushort2( ptr+4 ); // last  part of mantissa

	
	bool minusFlag = m1 < 0;
	if(  minusFlag  )							// number will be negative
	{
		m1 = ~m1;								// twos' complement mantissa
		m2 = ~m2 + 1;
		if( m2==0 )  m1 += 0x00000001;			// (carry 1 to m1)
			
		if( m1 <0 )  m1  = 1; 					// Must be 0x800000 = -1
	}											// (sign re-reversed later)
 	
    double v1 = m1 * c_2pM23;					// value from first part mantissa
    double v2 = m2 * c_2pM39;					// value from last  part mantissa
	
 												// Scale to characteristic and
    v1 = (v1+v2) * pow( 2.0, exp );				// set sign
	if( minusFlag ) v1 = -v1;
	
    return v1;
}

//page
//------------------------------------------------------------------------------
//  MiL1750_FP
//		- 4-byte float
//------------------------------------------------------------------------------

double MiL1750_FP( char* ptr )
{
    char buf[6];
         buf[4] = 0;  buf[5] = 0;
	 memcpy( buf, ptr, 4 );

    return MiL1750_EPFP( buf );
}

