//==============================================================================
//
// Copyright © 1999-2011, United States Government as represented by the 
// Administrator for The National Aeronautics and Space Administration.  
// All Rights Reserved.
//
// aqua_main.c- Aqua MODIS Direct Broadcast Attitude Data to GDAAC Level 1 Format
//
// 04 June 2001 Version 1.0
// 11 July 2002         1.1 - allow file names on command line
// 12 Aug  2002		1.2 - implement optional gbad packet size = 214 (AURA)
//			    - allow multiple APIDs (see packetfile)
//			    - configfile not mandatory
//
// 01 Nov  2002         1.3 - update C++ header files deprecated in 3.2 compiler
//			      e.g. <iostream.h> is now <iostream>
//			    - constrain desc. node to +/- pi. see satpasses.c.
//			    - constrain M in osculating keplerian elements.
//			    - remove miscellaneous dead code. 
//			    - add missing epochtime to ephemeris subheader
//			    - minor fixes to makefile in diaglib.
//
// 02 Apr  2003		1.4 - Add diag-3 code to isolate errors.
//
// 08 Aug  2003		1.5 - fix wrong cast, add to print_state list in satpasses
//					    - check for reversed times in repairstream.c
//					    - fix eof logic in repairstream.c
// 
// 27 Oct  2003		1.6 - allow aura packet id = 967
//
// 18 Nov  2003     	1.7 - fix negative number problem in aq_proc_Mil1750_EPFP.c
//
// 20 Mar  2006		1.8 - remove size_t typedef in aq_typedef_header.h
//						- add stackbuffer<T>:: to methods in aq_class_windowbuffer.h
//			    		- add windowbuffer<T>:: to aq_class_centeredwindowbuffer.h
//			    		- fix FormatConvert_long4 to work with 8byte long 
//						  aq_proc_formatconvet_x
//
// 22 Jul  2011		1.9 - made NORAD TLE files optional
//
//page
// This program converts Aqua or Aura MODIS Direct Broadcast Level 0 Ephemeris 
// and Attitude data to a subset of the Data Active Archive Center (DAAC) 
// Level 1 Format.  
//
// The input attitude data are in the Ground Based Attitude Determination Data 
// (GBAD) format. See 
//
//	"EOS PM-1 Spacecraft to EOS Ground System Interface Control Document",
//	GSFC 422-11-19-03, Section 5.5.4 Ground Based Attitude Determination 
//	Data
// 
// An input NORAD Two-Line Element File is optional.
// A configuration file is required.
//
// The output ephemeris and attitude files are in a subset of the DAAC Level 1 
// format.  See
//
//	"SDP Toolkit, User's Guide, Appendix L. Ephemeris and Attitude File 
//      Formats."
//
// Program execution command line:
//
//	aqua_main  [-configfile configfile_name]
//
//	configfile_name is an optional /path/configfileName of the configuration
// 	file. If not present, the configuration file is assumed to be named 
// 	"configfile" in the current directory. 
//
// Input Configuration File Contents:
//
//	(keyword)		(value)
//
//	-station		station receiving data
//	-version		
//	-packetfile		input GBAD packet file name
//	-noradfile		input NORAD Two-Line Element file name
//	-ephemerisfile		output ephemeris file name
//	-attitudefile		output attitude  file name
//	-noradsatname		input NORAD file satellite name
//	-sdpsatname		output ephemeris and attitude file sat. name
//	
// Output Ephemeris File
//
//	Spacecraft ephemeris in subset of DAAC Level 1 Format.
//
// Output Attitude File
//
//	Spacecraft attitude in subset of DAAC Level 1 Format.
//
//==============================================================================
//page

#include  <iostream>
#include  <time.h>
#include  <algorithm>

#include  "aq_const_header.h"
#include  "aq_class_configurationlist.h"
#include  "aq_class_configurationfile.h"
#include  "aq_class_cmd_line.h"
#include  "aq_class_cfg_list.h"
#include  "aq_class_packetfile.h"
#include  "aq_class_gbadpacket.h"
#include  "aq_class_noradfile.h"
#include  "orbitlib_class_NoradElements.h"

#include  "aq_class_ephemerissubheader.h"
#include  "aq_class_ephemerismetadata.h"
#include  "aq_class_commonSDPheader.h"
#include  "aq_class_attitudesubheader.h"
#include  "aq_class_attitudefile.h"
#include  "aq_class_ephemerisfile.h"

#include  "aq_proc_quaterniontoeuler.h"
#include  "timelib_class_datetime.h"
#include  "aq_proc_fileopencheck.h"
#include  "aq_proc_readcmdline.h"
#include  "aq_class_metafile.h"

#include  "aq_class_gbadpktprocessor.h"
#include  "aq_class_satpasses.h"
#include  "aq_class_orbitstuff.h"

#include  "aq_proc_CopyFiles.h"

//page
//-----------------------------------------------------------------------------
// ReadConfigFile- Read the configuration file
//
//	configuration file contains configurable parameters in 
//	"-keyword value" pairs.  
//	"aq_class_cfg_list.h" and "aq_class_cmdline.h" define valid keywords
//-----------------------------------------------------------------------------

void ReadConfigFile( configuration_list& cfgList, cmd_line& cmdline)
{

    configurationfile 
    cfgFile( cmdline.getvalue("-configfile").c_str() );
    FileOpenCheck( cfgFile );

    string key;
    string value;

    while( cfgFile.readline(key,value) )
    {
	if( !cfgList.setvalue(key,value) )
        {
	    cerr<<"\nBad Configuration File Keyword: "<<key<<endl;
	    exit(-1);
	}
    }

    if( !cfgFile.eof() ) 
    {
	cerr<<"\nConfiguration File read error"<<endl;
	exit(-1);
    }

    return;
}
 

//page
//-----------------------------------------------------------------------------
// GetEndPackets
//
// 	Get first and last packets from input file and check. If these are ok,
// 	probably file is valid. File must contain at least two packets.
//-----------------------------------------------------------------------------

void 
GetEndPackets( packetfile& pFile, gbadpacket& begPkt, gbadpacket& endPkt )
{
    pFile.ReadPacket( begPkt, pFile.BegPacket() );
    pFile.ReadPacket( endPkt, pFile.EndPacket() );

    if( pFile.BegPacket() == pFile.EndPacket() )
    {
	cerr<<"\nPacket File Empty"<<endl;
	exit(-1);
    }

    if( !pFile ) 
    {
	cerr<<"\nPacket File Read Error"<<endl;
	cerr<<"\nBegin Packet is:\n"<<begPkt<<endl;
	cerr<<"\nEnd   Packet is:\n"<<endPkt<<endl;
	exit(-1);
    }

    if( begPkt.PktSize()+6 != gbadpacket::pktsize )
    {
	cerr<<"\nBad first packet size: "<<begPkt.PktSize()<<endl;
	cerr<<'\n'<<begPkt<<endl;
	exit(-1);
    }
}


//-----------------------------------------------------------------------------
// GetNoradElements
//
//	Get NORAD Two-Line Element set from opened Norad File
//-----------------------------------------------------------------------------

bool GetNoradElements( noradfile& nfile, noradelements& noradtle )
{
    nfile.read( noradtle );
    
    if( !nfile )
		return false;
	else
		return true;
}
//page


//-----------------------------------------------------------------------------
// CopyEphemerisMetaData - write metadata footers to Ephemeris File
//
//	Get list of orbits (orbitstuff structs) assembled by 
//	"gbadpktprocessor", then loop thru list, using "satPass" to comput 
//	orbit parameters which are loaded into "ephemerismetadata" structs 
//	and written to file.
//	Boolean flag exists that determines if static values are to be used
//-----------------------------------------------------------------------------

void CopyEphemerisMetaData( 
     gbadpktprocessor&    pktprocessor, 
     const noradelements& noradtle, 
     ephemerisfile&       efile,
     const cfg_list&      configList,   // from configuration file
	 bool				  noradFileExists 	
)
{
    ephemerismetadata metaRcd;			// metadata record 
    satpasses satPass( noradtle );		// compute orbit parameters

					// diagnostic
    bool printPass = configList.getvalue("-diag_2" ) == (string)"yes" ;
    bool debug1    = configList.getvalue("-debug_1") == (string)"yes" ;

    const vector<orbitstuff>& orbits = pktprocessor.GetOrbitList();

    for( size_t k = 0; k<orbits.size(); k++)
    {
    	satPass.SetOrbit( orbits[k] );
		if( debug1 )   satPass.print_state();
		if( printPass) cerr<<"\ndiag_2 SatPass["<<k<<"]\n"<<satPass<<endl;
	
		//if norad file is present, use its data points
		if(noradFileExists)
		{
    		metaRcd.OrbitNo( 
						satPass.BegOrbitRev() );		
    		metaRcd.AscendingTime( 
						satPass.AscendingNodeTime() );
    		metaRcd.DescendingTime(
						satPass.DescendingNodeTime() );
    		metaRcd.DescendingLongitude( 
						satPass.DescendingNodeLongitude() );


		}
		//if norad file is not present
		else
		{
			//set data points to 1, except longitude
			//it is multiplied (to convert from rad to deg), 
			//so it will be set to 0 to not appear real
 	  		metaRcd.OrbitNo( 1 );		
    		metaRcd.AscendingTime( -999.99999999 );
    		metaRcd.DescendingTime( -999.99999999 );
			//this value is necessary
			//due to modisl1db performing its own
			//calculation based off this value
			//results in -999.0
    		metaRcd.DescendingLongitude( -17.43583922742335 );
		}

    	efile.write(metaRcd);
    }
}


//page
//-----------------------------------------------------------------------------
// void GetOrbitRevNos
//
// 	Get first and last Orbit Revolution Number
//-----------------------------------------------------------------------------

void GetOrbitRevNos(
    gbadpktprocessor     &processPkt, 
    const  noradelements &noradtle,
    size_t &begOrbitRev, 
    size_t &endOrbitRev
)
{
    const vector<orbitstuff>& orbitList = processPkt.GetOrbitList();
    size_t end = orbitList.size() - 1;

    satpasses satPass( noradtle );

    satPass.SetOrbit( orbitList[0] );
    begOrbitRev = satPass.BegOrbitRev();

    satPass.SetOrbit(  orbitList[end]);
    endOrbitRev = satPass.EndOrbitRev();

}

//-----------------------------------------------------------------------------
// GetBegKepler
//
//	Get approximate keplerian elements at ascending node of first orbit
//
//-----------------------------------------------------------------------------

array GetBegKepler( gbadpktprocessor &processPkt, const noradelements &noradtle)
{
    const orbitstuff begOrbit = processPkt.GetOrbitList()[0];
    
    satpasses begPass( noradtle );
    return begPass.SetOrbit( begOrbit ).BegKepler(); 
}



//page
//-----------------------------------------------------------------------------
// ListConfiguration
//
//	List parameters interpreted from configuration file
//
//-----------------------------------------------------------------------------

void ListConfiguration( const cmd_line& cmndLine, const cfg_list& configList)
{
    if( configList.getvalue("-listconfig") != (string)"yes") 
    return;

    cout<<"\nGBAD "<<configList.getvalue( "-noradsatname")
	<<' '<<date_time( time(0) ).ConvertDateToString()<<endl;

    cout<<"\nCommand Line Options---\n"<<cmndLine;
    cout<<"\nConfig. File Options---\n"<<configList<<endl;
}


//-----------------------------------------------------------------------------
// XferCmdLineValues
//
// Allow values on command line to override configfile.
//-----------------------------------------------------------------------------

static void XferCmdlineValues( cmd_line& cmndLine, cfg_list& cfgList )
{
    string xferValue;

    xferValue = cmndLine.getvalue("-listconfig");
    if(  xferValue != "" ) cfgList.setvalue("-listconfig", xferValue); 

    xferValue = cmndLine.getvalue("-packetfile");
    if(  xferValue != "" ) cfgList.setvalue("-packetfile", xferValue); 

    xferValue = cmndLine.getvalue("-noradfile");
    if(  xferValue != "" ) cfgList.setvalue("-noradfile", xferValue); 

    xferValue = cmndLine.getvalue("-ephemerisfile");
    if(  xferValue != "" ) cfgList.setvalue("-ephemerisfile", xferValue); 

    xferValue = cmndLine.getvalue("-attitudefile");
    if(  xferValue != "" ) cfgList.setvalue("-attitudefile", xferValue); 
}



//page
//=============================================================================
// main procedure
//
//=============================================================================

int main (int argc, char *argv[])
{	
    //-------------------------------------------------------------------------
    // Configure program
    //
    //-------------------------------------------------------------------------					
    cmd_line  cmndLine;
    ReadCmdLine( cmndLine, argc, argv );
						
    cfg_list  configList;
    ReadConfigFile( configList, cmndLine);

    XferCmdlineValues( cmndLine, configList ); 
    ListConfiguration( cmndLine, configList );

	//if there is a TLE file specified by user, this becomes true
	//otherwise, this value is false 
	bool noTLEfile = false;

    string noradSatName = configList.getvalue("-noradsatname");
    if( noradSatName == (string)"AURA" ) 
	{								// default is AQUA: 
		gbadpacket::pktsize = 214;	//	pktsize = 126;
		gbadpacket::apid 	= 967;  //  apid = 957
	} 

    if( configList.getvalue("-diag_1")==(string)"yes" )
	cerr<<"\ndiag_1 Pkt Read Size = "<<gbadpacket::pktsize<<endl;


    //-------------------------------------------------------------------------
    // open input files
    //
    //-------------------------------------------------------------------------

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"\nDiag 3- Begin Open Input Files"<<endl;

    packetfile 
    packetFile( configList.getvalue("-packetfile"), gbadpacket::apid); 
    FileOpenCheck( packetFile );

	if(configList.getvalue("-noradfile") == "noTLE")
	{
		noTLEfile = true;
		
		cout << "No TLE file specified. Using static values."; 
		cout << endl << endl;
	}

	noradfile 
	noradFile( configList.getvalue("-noradfile"), noradSatName );

	if(!noTLEfile)
	{ 
		FileOpenCheck( noradFile );
	}

    gbadpacket begPacket;
    gbadpacket endPacket;
    GetEndPackets( packetFile, begPacket, endPacket );     

    noradelements noradtle;

	//if norad File is present, fill noradtle instance with data 
	//from it
	if(!noTLEfile)
    	GetNoradElements( noradFile, noradtle );

    if( configList.getvalue("-diag_1")==(string)"yes" )
    {
	cerr<<"\ndiag_1 Beg Pkt"<<begPacket<<endl;
	cerr<<"\ndiag_1 End Pkt"<<endPacket<<endl;

	cerr<<"\ndiag_1 TLE "<<endl; 
	noradtle.Print(cerr) ;
    }


    //page
    //-------------------------------------------------------------------------
    // define headers and initialize with static info. (will be updated later)
    //
    //-------------------------------------------------------------------------
 
    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"\nDiag 3- Define Headers and Initialize with Static Info"<<endl;

    commonSDPheader SDPheader;
    SDPheader.SpacecraftID( configList.getvalue("-sdpsatname") );
	SDPheader.Source( configList.getvalue("-station") );
	SDPheader.Version( configList.getvalue("-version") );
	SDPheader.TimeRange( begPacket.TimeTag(), endPacket.TimeTag() );
    SDPheader.Interval( cst_interval );

    ephemerissubheader ephemSubhdr;
	ephemSubhdr.RefFrame( cst_keplerRefFrame );

	//if norad TLE file is present, use TAI from it
	//if not, use TAI of 1
	if(!noTLEfile)
		ephemSubhdr.EpochTimeTAI( 
					satpasses(noradtle).EpochTimeTAI93() );
    else 
	{
		ephemSubhdr.EpochTimeTAI(   -999.999999 );
	}

    attitudesubheader   attSubhdr;
    	const uint *order = cst_eulerAngleOrder;
	attSubhdr.AngleOrder( order[0], order[1], order[2] );


    //page
    //--------------------------------------------------------------------
    // Open Ephemeris File
    //--------------------------------------------------------------------

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Open Ephemeris and Attitude Files"<<endl;

    ephemerisfile ephemerisFile
    ( configList.getvalue("-ephemerisfile"), SDPheader, ephemSubhdr);

        FileOpenCheck( ephemerisFile );


    //-------------------------------------------------------------------------
    // Open Attitude File
    //-------------------------------------------------------------------------

    attitudefile 
    attitudeFile( configList.getvalue("-attitudefile"), SDPheader, attSubhdr);

        FileOpenCheck( attitudeFile );


    //-------------------------------------------------------------------------
    // Copy per-packet data to output files.  processPkt accumulates info. 
    //-------------------------------------------------------------------------

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Copy Packet Data to Output Files"<<endl;

    gbadpktprocessor  processPkt( packetFile ); 
    CopyFiles( processPkt, ephemerisFile, attitudeFile, configList );

	
    //-------------------------------------------------------------------------
    // Copy EphemerisMetaData
    //-------------------------------------------------------------------------

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Copy Ephemeris MetaData"<<endl;
	
	//if norad TLE file is present, call function with flag indicating 	   //presence of TLE file, with 1 as last parameter
	//if norad TLE file is not present, call function indicating
	//absence of file, with 0 as last parameter
	
	if(!noTLEfile)
    CopyEphemerisMetaData( processPkt, noradtle, 
						   ephemerisFile, configList, 1);
	else
	CopyEphemerisMetaData( processPkt, noradtle, 
						   ephemerisFile, configList, 0);
	

    //page
    //-------------------------------------------------------------------------
    // Update Header Info
    //-------------------------------------------------------------------------

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Update Header Info- GetOrbitRevNos- A"<<endl;

    size_t begOrbitRev;
    size_t endOrbitRev;
    GetOrbitRevNos( processPkt, noradtle, begOrbitRev, endOrbitRev );


    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Update Header Info- GetOrbitRevNos- B"<<endl;
 
    SDPheader.NoRecords( processPkt.RecordCount() );


    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Update Header Info- GetOrbitRevNos- C"<<endl;

	//if norad TLE file is present, get orbit revs from it
	//If not, use 1 1 1 as fill values.
	if(!noTLEfile)
    	ephemSubhdr.Orbits(begOrbitRev,endOrbitRev, 
					max( endOrbitRev-begOrbitRev, (size_t)1)  );
    else
   		ephemSubhdr.Orbits(1,1,1);

    if( configList.getvalue("-diag_3")==(string)"yes" )
	cerr<<"Diag 3- Udate Header Info- GetOrbitRevNos- D"<<endl;
	//If norad TLE file is present, use Kepler elements from it
	//If norad TLE file is not present, use fill values
	if(!noTLEfile)
    	ephemSubhdr.KeplerElements( 
			        GetBegKepler( processPkt,noradtle ) );
	else 
	{
		array begKeplerFill(6);
		begKeplerFill[0]=-999.999999;
		begKeplerFill[1]=-999.999999;
		begKeplerFill[2]=-999.999999;
		begKeplerFill[3]=-999.999999;
		begKeplerFill[4]=-999.999999;
		begKeplerFill[5]=-999.999999;
    	ephemSubhdr.KeplerElements( begKeplerFill );
	}

    if( configList.getvalue("-listconfig") == (string)"yes") 
        cout<<"\nnormal termination\n"<<endl;
    
}




























