//==============================================================================
// Ephemeris File Diagnostic - 
//
// 04 June 2001 Version 1.0
// 08 Aug  2003 Version 1.1 - make max record count message more clear
//
// DumpEphemeris  -file filename -beg [0] -cnt [all]
//
//==============================================================================

#include <iostream>
#include <fstream>
#include <iomanip>

#include "aq_typdef_header.h"
#include "aq_proc_printarray.h"
#include "aq_proc_fileopencheck.h"
#include "aq_class_commonSDPheader.h"
#include "aq_class_ephemerissubheader.h"
#include "aq_class_ephemerisrecord.h"
#include "aq_class_ephemerismetadata.h"
#include "aq_proc_readcmdline.h"
#include "aq_class_qualityflags.h"
#include "constlib_MathConstants.h"

double radToDeg = math_const_radiansToDegrees;


//------------------------------------------------------------------------------
// cfg_list
//
//------------------------------------------------------------------------------

class cfg_list : public configuration_list
{
public:
    cfg_list()
    {
        define( "-file"  );
        define( "-beg", "0" );
        define( "-cnt", "all" );
    }
};


//page
//------------------------------------------------------------------------
// main
//
//------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    cout<<"\nEphemeris File Diagnostic"<<endl;

    cfg_list cmdline;
    ReadCmdLine( cmdline, argc, argv );

    //-----------------------------------
    // Open Ephemeris File
    //-----------------------------------
    string fileName  = cmdline.getvalue( "-file" );

    ifstream ephemFile( fileName.c_str() );
    if( !ephemFile )
    {
	cerr<<"\nEphemeris File open failed: "<<fileName<<endl;
	return -1;
    }

    cout<<"Ephemeris File opened: "<<fileName<<endl;


    //-----------------------------------
    // Read SDP header  
    //-----------------------------------

    commonSDPheader SDPheader;
    ephemFile.read( SDPheader.DataPtr(), SDPheader.DataSize() );
    size_t tellpos = ephemFile.tellg();

    cout.setf( ios::fixed );
    cout<<endl;

    cout<<"\nSpacecraft = "<<SDPheader.SpacecraftID()
	<<"\nSource     = "<<SDPheader.Source()
	<<"\nVersion    = "<<SDPheader.Version()
        <<"\nTime Range = "<<SDPheader.TimeRange()
        <<"\nInterval   = "<<SDPheader.Interval()
        <<"\nNo. Records= "<<SDPheader.NoRecords()
	<<"\nNo. U. Refs= "<<SDPheader.NoURefs()
	<<"\n\n tellg() = "<<tellpos
	<<endl;

    //page
    //-----------------------------------
    // Read Ephemeris SubHeader
    //-----------------------------------

    ephemerissubheader ephemsubhdr;
    ephemFile.read( ephemsubhdr.DataPtr(), ephemsubhdr.DataSize() );
    tellpos = ephemFile.tellg();

    double TAI93epochSec = ephemsubhdr.EpochTimeTAI();
	double JD = 0;


	if (TAI93epochSec < 0)
		JD = -999.999999;
	else
		JD = TAI93epochSec/86400+2448988.5003125;

    cout<<endl;
    cout<<"\nBeg. Orbit = "<<setw(5)<<ephemsubhdr.Orbits()[0]
    	<<"\nEnd  Orbit = "<<setw(5)<<ephemsubhdr.Orbits()[1]
    	<<"\nOrbit Cnt. = "<<setw(5)<<ephemsubhdr.Orbits()[2]
	<<'\n'
        <<"\nRef. Frame = "<<ephemsubhdr.RefFrame()
	<<'\n'
	<<"\nKeplerian Elements"
	<<'\n'<<ephemsubhdr.KeplerElements()
	<<"\nTAI93 sec = "<<TAI93epochSec
	<<"  JD = "<<(JD)
	<<"\n\n tellg() = "<<tellpos
	<<endl;

     vector<float> 
     qaP = ephemsubhdr.qaParameters();
     cout<<"\nQA Parameters";
     cout.setf( ios::showpos );

     for( size_t pi = 0; pi<16; pi++)
	cout<<'\n'<<setw(2)<<pi<<' '<<qaP[pi];
     cout<<endl;

     qaP = ephemsubhdr.qaStatistics();
     cout<<"\nQA Statistics";

     for( size_t pi = 0; pi<4; pi++)
	cout<<'\n'<<setw(2)<<pi<<' '<<qaP[pi];
     cout<<endl;


    //-----------------------------------
    // No. of Universal References
    //-----------------------------------

    // **************************

    cout<<"\ntellg() before Universal Refs = "<<ephemFile.tellg()<<endl;

    ephemFile.seekg(SDPheader.NoURefs()*256, ios::cur);

    cout<<"tellg() after  Universal Refs = "<<ephemFile.tellg()<<endl;

    //page
    //-----------------------------------
    // Ephemeris Records
    //-----------------------------------

    size_t beg = atoi( cmdline.getvalue( "-beg" ).c_str() );

    size_t cnt = size_t(-1);
    size_t knt = 0;

    if( cmdline.getvalue( "-cnt" ) != (string)"all")
	cnt = atoi( cmdline.getvalue( "-cnt" ).c_str() );

    quality_flags qflags;
    ephemerisrecord ephemRec;

    cout<<"\nBeg Record = "<<beg<<endl;
	
    if( cnt != size_t(-1) ) cout<<"Rec Count  = "<<cnt<<endl;   // **
		else cout<<"Rec Count = all"<<endl; 
	
    cout<<"\nJulian Date, Position, Velocity QaFlag"<<endl;
    cout.unsetf( ios::showpos );

    double lastTime = 0;
    double thisTime = 0;
    size_t rec_cnt  = 0;

    for( size_t nrec = 0; nrec < SDPheader.NoRecords(); nrec++)
    {
	ephemFile.read( ephemRec.DataPtr(), ephemRec.DataSize() );
	rec_cnt++;

        if( nrec  <  beg ) continue;
	if( knt++ >= cnt ) continue;

	cout.precision(8);
	thisTime = ephemRec.time();

	if( thisTime <= lastTime )
	{
	    cout<<"\nTime Sequence Error "<<endl;
	}
	lastTime = thisTime;
	cout<<thisTime<<' ';

	cout.setf( ios::showpos );
	cout.precision(2);

	cout<<ephemRec.Position()<<' '<<ephemRec.Velocity()
	    <<' '<<hex<<setw(4)<<setfill('0');

	qflags.setqualityflags( ephemRec.QaFlag() );
	if( qflags.getqualityflags().any() ) cout<<'\n'<<qflags;
	cout<<endl;

	cout.unsetf( ios::showpos );
    }

    cout<<dec;
    cout<<"\ntellg() after ephemeris records = "<<ephemFile.tellg()<<endl;
    cout<<"  record count = "<<rec_cnt<<endl;

    //-----------------------------------
    // Ephemeris Metadata
    //-----------------------------------


    ephemerismetadata metadata;

    cout.precision(8);

    rec_cnt = 0;
    for( size_t m=0; m<ephemsubhdr.Orbits()[2]; m++)
    {
      ephemFile.read( metadata.DataPtr(), metadata.DataSize() );
      
      if( !ephemFile )
      {
	if( ephemFile.eof() ) cout<<"Error: eof bit set"<<endl;
	cout<<"file error"<<endl;

	break;
      }
       
	bool noTLE = false;
	if(metadata.AscendingTime() < 0)
		noTLE = true;

	double longitude = metadata.DescendingLongitude();
	double rad = longitude * radToDeg;
	if (noTLE)
	{
		longitude = -999.99999999;
		rad = -999.99999999;
	}

      cout<<"\nMetaData  "<<dec<<(++rec_cnt)<<"  \ttellg() = "<<ephemFile.tellg()
 	<<"\nOrbit No. = "<<metadata.OrbitNo()
	<<"\nAscending  Node Time = "<<metadata.AscendingTime()
	<<"\t TAI = "<<metadata.AscendingTimeTAI()
  	<<"\nDescending Node Time = "<<metadata.DescendingTime()
	<<"\nDescending Node Long.= "<<longitude
	<<" = "<<(rad)<<" Deg"
  	<<endl;
    }
}








