//==============================================================================
// GBAD Packet File Diagnostic - Dump a Packet in File
//
// 04 June 2001 Version 1.0
//
// Dump the specified packet in an APID 957 GBAD packet file. Keyword-value pairs
// may be in any order.  pkt value starts at 0.   Defaults are: -pkt 0 -sat AQUA.
//
// DumpPacket  -file file_name  -pkt [0]  -sat <AQUA|AURA>
//
//==============================================================================

#include <iostream>
#include <fstream>
#include <iomanip>

#include "aq_class_gbadpacket.h"
#include "aq_class_packetfile.h"
#include "aq_typdef_header.h"
#include "aq_proc_printarray.h"
#include "aq_proc_fileopencheck.h"
#include "aq_proc_readcmdline.h"
#include "timelib_class_datetime.h"


//------------------------------------------------------------------------------
// cfg_list
//
//------------------------------------------------------------------------------

class cfg_list : public configuration_list
{
public:
    cfg_list()
    {
        define( "-file"  );
	define( "-pkt", "0" );
	define( "-sat", "AQUA" );
    }
};


//------------------------------------------------------------------------------
// Dump File State
//
//------------------------------------------------------------------------------

void DumpState( packetfile& file )
{
    cout<<"\tEnd Packet = "<<file.EndPacket()<<endl;
    if( file.eof()  ) cout<<"\tEOF  Bit Set"<<endl;   
    if( file.bad()  ) cout<<"\tBAD  Bit Set"<<endl;   
    if( file.fail() ) cout<<"\tFAIL Bit Set"<<endl;   
}


//page
//------------------------------------------------------------------------
// main
//
//------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    cout<<"\nPacket File Diagnostic"<<endl;

    cfg_list cmdline;
    ReadCmdLine( cmdline, argc, argv );

    string fileName = cmdline.getvalue( "-file" );
    string satName  = cmdline.getvalue( "-sat"  );
    size_t pktno = atoi( cmdline.getvalue( "-pkt" ).c_str() );
    
    cout<<"packet number is  : "<<pktno<<endl;
    cout<<"Spacecraft name is: "<<satName<<endl;

    if( satName == (string)("AURA") ) gbadpacket::pktsize=214;

    //-----------------------------------
    // Open GBAD File
    //-----------------------------------

    packetfile 
    pktFile( fileName, gbadpacket::apid );
    FileOpenCheck( pktFile );

    cout<<"Packet File opened: "<<fileName<<endl;
    cout<<"APID "<<gbadpacket::apid
	<<" packet count: "<<pktFile.EndPacket()<<endl;

    if( pktFile.EndPacket() == 0 )
    {
	cout<<"No APID "<<gbadpacket::apid<<" packets found"<<endl;
	return -1;
    }


    //-----------------------------------
    // Read first GBAD Packet
    //-----------------------------------

    gbadpacket gPacket;
    
    if( !pktFile.ReadPacket( gPacket, pktno ) )
    {
        cout<<"\nRead Error on first Packet"<<endl;
	DumpState( pktFile );
    }

    char * dPtr = gPacket.DataPtr();
    size_t idx;


    //-----------------------------------
    // CCSDS Header
    //-----------------------------------
  
    cout<<"\nCCSDS Header     ";
    gPacket.Dump(cout,0,6);
    cout<<"  (hex)"<<endl;

    cout<<"\n"<<dec<<(packet)gPacket<<endl;

    //page
    //-----------------------------------
    // CCSDS Time Stamp
    //-----------------------------------
  
    cout<<"\nCCSDS Time Stamp\n     ";
    gPacket.Dump(cout,6,14);
    cout<<"\t(hex)"<<endl;

    cout<<"     "<<dec<<gPacket.CoarseTime()<<"\t\t        (TAI)"<<endl;
    cout<<"     "<<dec<<gPacket.FineTime()<<" = "<<(gPacket.FineTime()*15.2e-6)
	<<endl;

    cout.setf( ios::fixed );
    cout.precision(8);

    cout<<"     "<<dec<<gPacket.TimeTag()<<"\t\t(JD )"<<endl;

    date_time utcdate( gPacket.TimeTag() );
    cout<<"     "<<utcdate.ConvertDateToString()<<endl;

    //-----------------------------------
    // Position
    //-----------------------------------
        
    cout<<"\nPosition\n     ";
	gPacket.Dump(cout,14,20);
 
    cout<<"\n     ";
	gPacket.Dump(cout,20,26);

    cout<<"\n     ";	
	gPacket.Dump(cout,26,32);
 
    cout<<"\n     ";
    cout<<dec<<gPacket.Position()<<endl;


    //-----------------------------------
    // Velocity
    //-----------------------------------
        
    cout<<"\nVelocity\n     ";
	gPacket.Dump(cout,32,38);

    cout<<"\n     ";
	gPacket.Dump(cout,38,44);

    cout<<"\n     ";
	gPacket.Dump(cout,44,50);

    cout<<"\n     ";
    cout<<dec<<gPacket.Velocity()<<endl;


    //page
    //-----------------------------------
    // Angle (quaternion)
    //-----------------------------------
 
    cout<<"\nAngle\n     ";
	gPacket.Dump(cout,50,56);

    cout<<"\n     ";
	gPacket.Dump(cout,56,62);

    cout<<"\n     ";
	gPacket.Dump(cout,62,68);
 
    cout<<"\n     ";
	gPacket.Dump(cout,68,74);

    cout<<"\n     ";
    cout<<dec<<gPacket.Attitude()<<endl;


    //-----------------------------------
    // Angle Rate
    //-----------------------------------
 
    cout<<"\nAngle Rate\n     ";
	gPacket.Dump(cout,74,78);
 
    cout<<"\n     ";
 	gPacket.Dump(cout,78,82);

    cout<<"\n     ";
	gPacket.Dump(cout,82,86);

    cout<<"\n     ";
    cout<<dec<<gPacket.AngleRates()<<endl;

}








