//========================================================================
// mathlib_class_Matrix.c
//
// 09 Dec 1999 Version 1.0
// 21 Aug 2000 version 1.1 - add vector<double>
//
// This implements a simple matrix class which provides most common matrix
// arithematic operations.  This class also provides row, column, and subarray 
// operations.
//
// If checking is enabled, this class throws a MatrixError exception when
// subscripts are out of range.  Checking is enabled by: matric::check=true;
//
//  MatrixError members:
//
//  string _message;			// operation causing error
//  size_t _mtx1_row, _mtx1_col;	// first  matrix
//  size_t _mtx2_row, _mtx2_col;	// second matrix
//
//
//========================================================================

//page
/* -----------------------------------------------------------------------

-- Construction-
   A matrix can be constructed as follows:

 	matrix A (noRows, noCols);	
	matrix A (noRows, noCols, defaultValue);

	double array[3][3] = {11,12, 13, 21, 22, 23, 31,32,33};
	matrix A (noRows, noCols, array); 

        matrix A (noRows, noCols, vector<double> vectorList);

	matrix A = matrix(noRows, noCols).getarray(a11,a12,a13,a21,...);


-- Row and Column operations:

	A.row(a_row) = B;	      copy B matrix to row of A
	A.row(a_row) = C.row(c_row);  copy row of C to rwo of A
	D = F.row(f_row);   	      copy row of F to matrix D

	column operations are similar.
	

-- Subarray operations:

	A.subarray(begin_row,begin_col) = B;
				      copy matrix B to subarray of A

	C = A.subarray(begin_row, begin_col, no_rows, no_cols);
  				      copy subarray of A to matrix C
//page
-- Matrix operations:
	(capital letters are matrices, lower case are scalers)

	columns = A.colsize();	 	// get number of columns
	rows    = A.rowsize();		// get number of rows

	M = A.getarray (val, val,...);
      	M = A.getarray (ptr);		// get and put elements to c-style
	A.putarray (ptr);		// arrays- double X[i][j];

      	M = A.getarray (vector<double>& list); // 2nd subscript advances most 
	A.putarray (vector<double>& list);     // rapidly
	vector<double> = A.putarray();

        A = (*doublePtr)(c-style array);
        A = (vector<double>);

	s = A(row,col);	A(row,col) = s; // access element of matrix

	A = B ;				// assign one matrix to another  
	A = s ;				// assign scaler to all entries

	A = B + C;			// add two matricies
	A = B + s;  A = s + B;		// add scaler to each matrix element
	A =+B;				// monadic operator
	A +=B;
	A -=s;
					// same for subtraction

	A = B * C;			// matrix (row by column) multiply
	A = B * s;  A = s * B;		// multiply every element by a scaler				

	A = B / s; 			// divide each element by a scaler
	A /= s;


	A = B++;    A = transpose(B);	// matrix transpose

	A = B.identity();		// set B to identity and return B

	A = product(B,C);		// element by element multiply
	s = A.sum();			// sum of all elements

        s = A.max(&i,&j);		// maximum, minimum elements and size_t
        s = A.min(&i,&j);		// indicies.
	s = A.max();
	s = A.min();

	os<<A;				// print elements of matrix
*/
//------------------------------------------------------------------
//page

#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "mathlib_class_Matrix.h"

bool matrix::check = false;		// matrix class error checking
					// disabled

//------------------------------------------------------------------------
// Internal proc-  Assert two matrices must be equal in row and col size
// 
//------------------------------------------------------------------------

static void Assert_M1eqM2
     (const string& msg, size_t row1, size_t col1, size_t row2, size_t col2)
{
     if (row1!=row2 || col1!=col2) throw MatrixError (msg, row1, col1, row2, col2);
}


//------------------------------------------------------------------------
// Internal proc-  Assert first matrix must contain the second
// 
//------------------------------------------------------------------------

static void Assert_M1geM2
     (const string& msg, size_t row1, size_t col1, size_t row2, size_t col2)
{
     if (row1<row2 || col1<col2) throw MatrixError (msg, row1, col1, row2, col2);
}




//------------------------------------------------------------------------
// _initial - internal procedure to initialize this instance.
//
//------------------------------------------------------------------------

void matrix::_initial (size_t rows, size_t cols, void *ptr)
{
    if (check) Assert_M1geM2 ("matrix constructor", rows, cols, 1, 1);

    _rows = rows;
    _cols = cols;

    _arrayPtr = new double [rows*cols];

    if (ptr) memcpy (_arrayPtr, ptr, rows*cols*sizeof(double));
}


//page
//------------------------------------------------------------------------
// _value - internal procedure to access entry by row and column. First 
//	    entry is (1,1).
//
//------------------------------------------------------------------------

double&  matrix::_value (size_t row, size_t col) const
{
    return _arrayPtr [(row-1)*_cols+col-1];
}


//------------------------------------------------------------------------
// constructors for internal operations
//------------------------------------------------------------------------

matrix::matrix (const _row_matrix& rm)		// row_matrix to matrix
{
    _initial (1, rm._mtx._cols);

    for (int i=0; i<rm._mtx._cols; i++) 
	_arrayPtr[i] = rm._mtx._value(rm._row,i+1);
}

matrix::matrix (const _col_matrix& rm)		// col_matrix to matrix
{
    _initial (rm._mtx._rows, 1);

    for (int i=0; i<rm._mtx._rows; i++) 
	_arrayPtr[i] = rm._mtx._value(i+1,rm._col);
}


//page
//------------------------------------------------------------------------
// constructors for public use
//------------------------------------------------------------------------

matrix::matrix (size_t rows, size_t cols, double value) 
{
    _initial (rows,cols);
    for (int i=0; i<rows*cols; i++) _arrayPtr[i] = value;
}

matrix::matrix (const matrix& mtx)
{
    _initial (mtx._rows, mtx._cols, mtx._arrayPtr);
}

matrix::matrix (size_t rows, size_t cols, double list[])
{
    _initial (rows,cols,list);
}

matrix::matrix (size_t rows, size_t cols, const vector<double>& vec)
{
    _initial (rows,cols);

    if (check && rows*cols != vec.size() )
	throw MatrixError ("getarray", _rows, _cols, vec.size(), 0);

    for (size_t i=0; i<rows*cols; i++) _arrayPtr[i] = vec[i];
}


//------------------------------------------------------------------------
// Destructor
//------------------------------------------------------------------------

 matrix::~matrix()
{
    delete [] _arrayPtr;
}


//------------------------------------------------------------------------
// Array Index operator - first element is (1,1)
//------------------------------------------------------------------------

double& matrix::operator() (size_t row, size_t col) const 
{
    if (check)
    {
        if (row==0 || col==0) throw MatrixError("zero index",row,col,1,1);
	Assert_M1geM2 ("Matrix index \"()\"", _rows, _cols, row, col);
    }

    return _value (row,col);
}


//page
//------------------------------------------------------------------------
// assignment operators  (=)
//------------------------------------------------------------------------

matrix& matrix::operator= (const matrix& m)
{
    if (check) 
    Assert_M1eqM2("Matrix assignment \"=\"", _rows, _cols, m._rows, m._cols);

    memcpy (_arrayPtr, m._arrayPtr, _rows*_cols*sizeof(double));
    return *this;
}

matrix& matrix::operator= (double v)
{
    for (int i=0; i<_rows*_cols; i++) _arrayPtr[i] = v;
    return *this;
}

matrix& matrix::operator= (const double arrayPtr[])
{
    memcpy (_arrayPtr, arrayPtr, _rows*_cols*sizeof(double));
    return *this;
}

matrix& matrix::operator= (const vector<double>& vec)
{
    getarray(vec);
    return *this;
}


//------------------------------------------------------------------------
// get and put to C-style arrays-  double X [i][j]
//------------------------------------------------------------------------

matrix& matrix::getarray (const double arrayPtr[])
{
    for (int i=0; i<_rows*_cols; i++) _arrayPtr[i] = ((double*)arrayPtr)[i];
    return *this; 
}

void matrix::putarray (double arrayPtr[]) const
{
    for (int i=0; i<_rows*_cols; i++) ((double*)arrayPtr)[i] = _arrayPtr[i];
}


//page
matrix& matrix::getarray( double value, ... )
{
    va_list   ap;
    va_start( ap, value );

    _arrayPtr[0] = value;

    for( size_t i=1; i<_rows*_cols; i++ )
    {
	double p = va_arg( ap, double);
	_arrayPtr[i] = p;
    }

    va_end(ap);

    return *this;
}


//------------------------------------------------------------------------
// get and put to vector containers.  Second subscript advances most rapidly
//------------------------------------------------------------------------

matrix&  matrix::getarray (const vector<double>& vec)
{
    if (check && _rows*_cols != vec.size() )
	throw MatrixError ("getarray", _rows, _cols, vec.size(), 0);

    for (size_t i=0; i<_rows*_cols; i++) _arrayPtr[i] = vec[i];
    return *this; 
}

void matrix::putarray (vector<double>& vec) const
{
    vec.clear();
    for (int i=0; i<_rows*_cols; i++) vec.push_back( _arrayPtr[i] );
}

vector<double> matrix::putarray() const
{
    vector<double> temp;
    for (int i=0; i<_rows*_cols; i++) temp.push_back( _arrayPtr[i] );
    return temp;
}


//------------------------------------------------------------------------
// sum - return sum of matrix elements
//------------------------------------------------------------------------

double matrix::sum() const
{ 
    double sum = 0.;
    for (int i=0; i<_rows*_cols; i++) sum += _arrayPtr[i];
    return sum;
}


//page
//------------------------------------------------------------------------
// Maximum element 
//------------------------------------------------------------------------

double  matrix::max (size_t *rowPtr, size_t  *colPtr) const 
{
    double max = 0.;
    size_t idx = 0 ;

    for (int i=0; i<_rows*_cols; i++) 
    {
	if (_arrayPtr[i]>max) 
	{
	    max = _arrayPtr[i];
	    idx = i;
	}
    }

    if (rowPtr) *rowPtr = idx/_cols + 1;
    if (colPtr) *colPtr = idx-(*rowPtr-1)*_cols+1;

    return max;
}


//------------------------------------------------------------------------
// Minimum Element
//------------------------------------------------------------------------

double  matrix::min(size_t *rowPtr, size_t  *colPtr) const 
{
    double min = DBL_MAX;
    size_t idx = 0 ;

    for (int i=0; i<_rows*_cols; i++) 
    {
	if (_arrayPtr[i]<min) 
	{
	    min = _arrayPtr[i];
	    idx = i;
	}
    }

    if (rowPtr) *rowPtr = idx/_cols + 1;
    if (colPtr) *colPtr = idx-(*rowPtr-1)*_cols+1;

    return min;
}


//page
//------------------------------------------------------------------------
// addition operators (+)  Add scaler to Matrix
//------------------------------------------------------------------------

matrix  operator+ (const matrix& m, double v)
{
    matrix t = m;
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] += v;
    return t;
}

matrix  operator+ (double v, const matrix& m)
{
    return m+v;
}

matrix operator+= ( matrix& m, double v)
{
    return m = m + v;
}

//------------------------------------------------------------------------
// addition operators (+)  Add two Matricies
//------------------------------------------------------------------------

matrix  operator+ (const matrix& m, const matrix& n)
{
    if (matrix::check) 
    Assert_M1eqM2 ("Matrix addition \"+\"", m._rows, m._cols, n._rows, n._cols);

    matrix t = m;
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] += n._arrayPtr[i];
    return t;	
}

matrix operator+= ( matrix& m, const matrix& n)
{
    return m = m + n;
}


//------------------------------------------------------------------------
// addition operators (+)  monadic signs
//------------------------------------------------------------------------

const matrix&  operator+ (const matrix& m)
{
    return m;
}

matrix&  operator+ (matrix& m)
{
    return m;
}


//page
//------------------------------------------------------------------------
// subtraction operators (-)  scalers
//------------------------------------------------------------------------

matrix  operator- (const matrix& m, double v)
{
    matrix t = m;
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] -= v;
    return t;
}

matrix  operator- (double v, const matrix& m)
{
    matrix t(m._rows, m._cols);
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] = v - m._arrayPtr[i];
    return t;
}


matrix operator-= ( matrix& m, double v)
{
    return m = m - v;
}


//------------------------------------------------------------------------
// subtraction operators (-)  Subtract matrices
//------------------------------------------------------------------------

matrix  operator- (const matrix& m, const matrix& n)
{
    if (matrix::check) 
    Assert_M1eqM2 ("Matrix subtraction \"-\"", m._rows, m._cols, n._rows, n._cols);

    matrix t = m;
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] -= n._arrayPtr[i];
    return t;	
}

const matrix  operator- (const matrix& m)
{
    const matrix t(m._rows, m._cols);

    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] = - m._arrayPtr[i];
    return t;
}


matrix operator-= ( matrix& m, const matrix& n)
{
    return m = m - n;
}

//page
//------------------------------------------------------------------------
// subtraction operators (-)  monadic sign
//------------------------------------------------------------------------

matrix  operator- (matrix& m)
{
    matrix t(m._rows, m._cols);

    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] = - m._arrayPtr[i];
    return t;

}


//------------------------------------------------------------------------
// multiplication operators (*) elements X scaler
//------------------------------------------------------------------------

matrix  operator* (const matrix& m, double v)
{
    matrix t = m;
    for (int i=0; i<t._rows*t._cols; i++) t._arrayPtr[i] *= v;
    return t;
}

matrix  operator* (double v, const matrix& m)
{
    return m*v;
}


//------------------------------------------------------------------------
// multiplication operators (*) row by column matrix multiply
//------------------------------------------------------------------------

matrix  operator* (const matrix& a, const matrix& b)
{
    if (matrix::check && a._cols!=b._rows) 
    throw  MatrixError ("Matrix multiply \"*\"", a._rows, a._cols, b._rows, b._cols);

    matrix t(a._rows, b._cols);
	
    for (int i=1; i<=a._rows; i++)
    for (int j=1; j<=b._cols; j++)
    for (int k=1; k<=a._cols; k++)
  	t._value(i,j) = a._value(i,k) * b._value(k,j) + t._value(i,j);

    return t;
}


//page
//------------------------------------------------------------------------
// Product of element by element multiplication
//------------------------------------------------------------------------

matrix product (const matrix& a, const matrix& b)
{
    if (matrix::check) 
    Assert_M1eqM2 ("Matrix product", a._rows, a._cols, b._rows, b._cols);

    matrix t (a._rows, a._cols);

    for (int i=0; i<a._rows*a._cols; i++) 
	t._arrayPtr[i] = a._arrayPtr[i]*b._arrayPtr[i];

    return t;
}


//------------------------------------------------------------------------
// identity matrix
//------------------------------------------------------------------------

    matrix& matrix::identity()
    {
	for (int i=0; i<_rows*_cols; i++) _arrayPtr[i]=0.;

	for (int j=1; j<= (_rows<_cols ? _rows : _cols); j++)
	_value(j,j)=1.0;

	return *this;
    }


//------------------------------------------------------------------------
// Division by a constant
//------------------------------------------------------------------------

    matrix  operator/ (const matrix& m, double v)
    {
	return m*(1.0L/v);
    }

    matrix  operator/= (matrix& m, double v)
    {
	return m = m * (1.0L/v);
    }


//page
//------------------------------------------------------------------------
// matrix transpose
//------------------------------------------------------------------------

    matrix  transpose (const matrix& m)
    {
	matrix t (m._cols, m._rows);

	for (int i=1; i<=t._rows; i++)
	for (int j=1; j<=t._cols; j++)
	    t._value(i,j) = m._value(j,i);
 
	return t;
    }


matrix  matrix::operator++ (int) const
{
	return transpose (*this);
}


//------------------------------------------------------------------------
// Build Row Matrix	(for left or right of assignment)
//------------------------------------------------------------------------

_row_matrix  matrix::row (size_t row)
{
    return _row_matrix (*this, row);
}


//------------------------------------------------------------------------
// Build Column Matrix  (for left or right of assignment)
//------------------------------------------------------------------------

_col_matrix  matrix::col (size_t col)
{
    return _col_matrix (*this, col);
}


//page
//------------------------------------------------------------------------
// Build Subarray for right side of assignment
//------------------------------------------------------------------------

matrix matrix::subarray 
(size_t row, size_t col, size_t noRows, size_t noCols) const
{
    if (check) Assert_M1geM2 ("subarray", _rows, _cols, row+noRows-1, col+noCols-1);

    matrix t(noRows,noCols);
    row--;
    col--;
	
    for (int i=1; i<=noRows; i++)
    for (int j=1; j<=noCols; j++)
	t._value(i,j) = _value(i+row, j+col);
	
    return t;
}


//------------------------------------------------------------------------
// Build Subarray for left side of assignment
//------------------------------------------------------------------------

_sub_matrix matrix::subarray (size_t row, size_t col)
{
    return _sub_matrix (*this, row, col);
}


//-----------------------------------------------------------------------
// Row  Constructor
//-----------------------------------------------------------------------
 
_row_matrix::_row_matrix (matrix& m, size_t row) : _mtx(m), _row(row) 
{ 
  if (matrix::check && _row > m._rows)
  {
	throw MatrixError ("row constructor", m._rows, m._cols, row, 0);
  }
}


//page
//------------------------------------------------------------------------
// Row assignment (=) from matrix
//------------------------------------------------------------------------

matrix& _row_matrix::operator= (const matrix& m)
{
    if (matrix::check && m._rows !=1  || m._cols != _mtx._cols)
    {
	throw MatrixError ("row assignment", _mtx._rows, _mtx._cols, m._rows, m._cols);
    }

    for (int i=1; i<=_mtx._cols; i++) _mtx._value(_row,i) = m._value(1,i);
    return _mtx;
}


//------------------------------------------------------------------------
// Row assignment (=) from row
//------------------------------------------------------------------------
matrix& _row_matrix::operator= (const _row_matrix m)
{
    if (matrix::check && m._mtx._cols != _mtx._cols)
    {
	throw MatrixError ("=row assignment", _mtx._rows, _mtx._cols, m._mtx._rows, m._mtx._cols);
    }

    for (int i=1; i<=_mtx._cols; i++) _mtx._value(_row,i) = m._mtx._value(m._row,i);
    return _mtx;
}


//-----------------------------------------------------------------------
// Column  Constructor
//-----------------------------------------------------------------------

_col_matrix::_col_matrix (matrix& m, size_t col) : _mtx(m), _col(col) 
{
  if (matrix::check && _col > m._cols)
  {
	throw MatrixError ("row constructor", m._rows, m._cols, 0, col);
  }
}


//page
//------------------------------------------------------------------------
// column assignment (=) from matrix 
//------------------------------------------------------------------------

matrix& _col_matrix::operator= (const matrix& m)
{
    if (matrix::check && m._cols !=1 || m._rows != _mtx._rows)
    {
	throw MatrixError ("=col assignment", _mtx._rows, _mtx._cols, m._rows, m._cols);
    }
 
    for (int i=1; i<=_mtx._rows; i++) _mtx._value(i,_col) = m._value(i,1);
    return _mtx;
}


//------------------------------------------------------------------------
// column assignment (=) from column 
//------------------------------------------------------------------------

matrix& _col_matrix::operator= (const _col_matrix m)
{
    if (matrix::check && m._mtx._rows != _mtx._rows)
    {
	throw MatrixError 
	("=row assignment", _mtx._rows, _mtx._cols, m._mtx._rows, m._mtx._cols);
    }

    for (int i=1; i<=_mtx._rows; i++) _mtx._value(i,_col) = m._mtx._value(i,m._col);
    return _mtx;
}


//------------------------------------------------------------------------
// subarray constructor
//------------------------------------------------------------------------

_sub_matrix::_sub_matrix (matrix& m, size_t begRow,  size_t begCol)
    : _mtx(m), _beginRow(begRow), _beginCol(begCol) 
{
    if (matrix::check && begRow>m._rows || begCol>m._cols)
    {
	throw MatrixError 
	("subarray constructor", m._rows, m._cols, begRow, begCol);
    }
}


//page
//------------------------------------------------------------------------
// subarray assignment (=) from matrix
//------------------------------------------------------------------------

matrix& _sub_matrix::operator= (const matrix& m)
{
    if (matrix::check &&_beginRow+m._rows-1 > _mtx._rows || 
	_beginCol+m._cols-1 > _mtx._cols)
    {
	throw MatrixError 
	("subarray assignment", _mtx._rows, _mtx._cols, m._rows, m._cols);
    }

    for (int i=0; i<m._rows; i++)
    for (int j=0; j<m._cols; j++)
	_mtx._value(i+_beginRow, j+_beginCol) = m._value(i+1,j+1);

    return _mtx;
}


//------------------------------------------------------------------------
// print matrix
//------------------------------------------------------------------------

ostream& operator<< (ostream& os, const matrix& m)
{
    os<<'\n';

    for (int i=1; i<=m._rows; i++)
    {
   	for (int j=1; j<=m._cols; j++)
            os<<m._value(i,j)<<' ';    

	os<<'\n';
    }

    return os;
}

