//==============================================================================
// orbitlib_class_NoradElements.c
//
// 29 Feb 2000 	Version 1.0
// 27 Jun 2002          1.1 fix format prob with rev no. 
//
//      See the United States Air Force Aerospace Defense Command Document, 
//      "Project Space Track, Models for propagation of NORAD Element Sets", 
//	December 1980.
// 
//
// This class provides NORAD Element Set Conversions. Unit conversions are those
// found in the original Driver for the NORAD Models.
//
//	angles are positive principle angles. 
//    
//	V[0] = XNO    -  mean motion     line 2, col 53-63      radians/minute
//	V[1] = EO     -  eccentricity    line 2, col 27-33	none
// 	V[2] = XINCL  -  inclination     line 2, col 09-16      rad.
//  	V[3] = XNODEO -  right ascen.    line 2, col 18-25	rad.
//	V[4] = OMEGAO -  arg. of perigee line 2, col 35-42	rad.
//      V[5] = XMO    -  mean anomaly	 line 2, col 44-51	rad.
//	V[6] = BSTAR  -  drag term	 line 1, col 54-61	earthradii**-1
//
//	not required by  SGP4 model:
//
//	V[7] = XNDT2O -  		 line 1, col 34-43      radians/min**2
//	V[8] = XNDD6O -			 line 1, col 45-52       
//
//      V[9] = 		 satellite no.   line 1, col 03-07
//      V[10]= 		 year            line 1, col 19-20
//      V[11]=           day             line 1, col 21-32
//	V[12]=           epoch rev       line 2, col 64-68
//
//==============================================================================

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "orbitlib_class_NoradElements.h"
#include "constlib_MathConstants.h"

static const double& DtR   = math_const_degreesToRadians;
static const double& twoPi = math_const_twoPi;


//page
//------------------------------------------------------------------------------
// Compute Checksum - 
//
//	add only digits(0-9), counting minus_sign(-) as 1, and
// 	return last digit
//
//------------------------------------------------------------------------------

static char CheckSum(const char* buffer)
{
    size_t checksum = 0;

    for( size_t i=0; i<68; i++)
    {
	if ( isdigit(buffer[i]) )  checksum += buffer[i]-48;
	if ( buffer[i] == '-'   )  checksum++;
//	cout<<i<<"("<<buffer[i]<<") "<<checksum<<endl;
    } 

    return (char)((checksum%10) + 48); 
}


//page
//------------------------------------------------------------------------------
// Constructors
//
//------------------------------------------------------------------------------

noradelements::noradelements () : V(13) 
{
    checksumValid = true;
}


noradelements::noradelements ( const string& line1, const string& line2 ) : V(13)
{
    Set(line1,line2);
}


noradelements::noradelements ( const vector<double>& _V )
{
    Set(_V); 
}


noradelements::noradelements( const noradelements&  N )
{
    Set( N );
}

    
//------------------------------------------------------------------------------
// Set from  keplerian element list
//
//------------------------------------------------------------------------------

noradelements&  noradelements::Set( const vector<double>& _V )
{
    V = _V;
    checksumValid = true;
    return *this;
}
    

//------------------------------------------------------------------------------
// Set from another noradelement object
//
//------------------------------------------------------------------------------

noradelements&  noradelements::Set( const noradelements&  N )
{
    V = N.V;
    checksumValid = N.checksumValid;
    return *this;
}

  
//page
//------------------------------------------------------------------------------
// Convert from String
//
//------------------------------------------------------------------------------

noradelements&  noradelements::Set( const string& line1, const string& line2 )
{
    size_t line1No;
    int    v8exp;
    int    v6exp;
    char   chksum1;
    char   chksum2;

    sscanf( line1.c_str(), 
	"%1d%*1c%5lf%*10c%2lf%12lf%*1c%10lf%*1c%6lf%2d%*1c%6lf%2d%*7c%1c", 
	&line1No, &V[9], &V[10], &V[11], &V[7], &V[8], &v8exp, &V[6], &v6exp,
        &chksum1); 

    V[8] =  V[8] * 1e-5 * pow( 10.0, v8exp );
    V[6] =  V[6] * 1e-5 * pow( 10.0, v6exp );

    size_t line2No;

    sscanf( line2.c_str(),
	"%1d%*7c%8lf%*1c%8lf%*1c%7lf%*1c%8lf%*1c%8lf%*1c%11lf%",
	&line2No, &V[2], &V[3], &V[1], &V[4], &V[5], &V[0] );

    string revno(line2,63,5);
    V[12] = atof( revno.c_str() );

    chksum2 = line2[68];

    V[0] *= twoPi/1440.L;
    V[1] *= 1e-7;
    V[2] *= DtR;
    V[3] *= DtR;
    V[4] *= DtR;
    V[5] *= DtR;
    
    V[7] *= twoPi / 1440.0L / 1440.0L;		// rev/Day**2 to rad/min**2
    V[8] *= twoPi / 1440.0L / 1440.0L /1440.0L;

    checksumValid = 
       (CheckSum(line1.c_str())==chksum1) && (CheckSum(line2.c_str())==chksum2);

    return *this;
}


//page
//------------------------------------------------------------------------------
// Convert to String
//
//------------------------------------------------------------------------------

void noradelements::Get(string& line1, string& line2) const
{
    char buffer[81];
						
    double X7 = V[7] * 1440.0L * 1440.0L / twoPi;  // rad/min**2 to rev/day**2
    double X8 = V[8] * 1440.0L * 1440.0L * 1440.0L / twoPi;

    int  V7 = abs( (int)(X7*1e8) );			// XNDT20
    char S7 = (X7 >= 0.) ? '+' : '-';

    char S8 = (X8 >= 0.) ? '+' : '-';			// XNDD60
    int  E8 = (int)(  (X8 == 0.) ? 0.0 : log10(X8)  );
    int  V8 = abs(   (int)(X8 * 1e5 * pow(10.0,-E8))   );

    char S6 = (V[6] >= 0.) ? '+' : '-';			// Bstar
    int  E6 = (int)(  (V[6] == 0.) ? 0.0 : log10( fabs(V[6]) )  );
    int  V6 =  abs(  (int)(V[6] * 1e5 * pow(10.0,-E6))  );

 
    sprintf( buffer, 
       "1 %5d           %02d%012.8lf %c.%08d %c%05d%+2d %c%05d%+2d                   ", 
       (int)V[9], (int)V[10], V[11], S7,V7, S8,V8,E8, S6,V6,E6 );

    buffer[68] = CheckSum( buffer );
    line1 = (string)buffer;

    double V0 = V[0]*1440.0L/twoPi;
 
    sprintf( buffer, 
        "2 %5d %8.4lf %8.4lf %07d %8.4lf %8.4lf %11.8lf%5d            ", 
        (int)V[9], V[2]/DtR, V[3]/DtR, (int)(V[1]*1e7), V[4]/DtR, V[5]/DtR,  
        V0, (int)V[12] );

    buffer[68] = CheckSum( buffer );
    line2 = (string)buffer;
}


//page
//------------------------------------------------------------------------------
// Get List
//
//------------------------------------------------------------------------------

const vector<double> noradelements::Get()  const
{
    return V;
}


//------------------------------------------------------------------------------
// Print two liner
//
//------------------------------------------------------------------------------

void noradelements::Print( ostream& os) const
{
   string line1;
   string line2;

   Get(line1,line2);
   cout<<line1<<'\n'<<line2<<endl;
}


//------------------------------------------------------------------------------
// validity check
//
//------------------------------------------------------------------------------

bool noradelements::operator!() const
{
    return checksumValid;
}



