//------------------------------------------------------------------------------
//
// orbitlib_proc_ConvertKeplerToRectangular.c - 
//
//
//      Computes geocentric inertial rectangular elements 
//      from osculating keplerian elements. The implementation follows "Brouwer-
//      Lyaddane Orbit Generator Routine", X-553-70-223, E. A. Galbreath, 
//      June 1970.
//
//      input: 
//
//	V[0] = A	semi major axis	  (meters)
//      V[1] = e	eccentricity
//      V[2] = i	incliniation	  (radians)
//      V[3] = N	Ascending Node    (radians)
//      V[4] = w	Angle of Perigee  (radians)
//      V[5] = m 	Mean Anomaly	  (radians)
//
//      returns:
//
//	V[0] = x  	(meters)
//	V[1] = y 	(meters)
//	V[2] = z	(meters)
//	V[3] = xdot 	(meters/sec)
//	V[4] = ydot 	(meters/sec)
//	V[5] = zdot 	(meters/sec)
//
//------------------------------------------------------------------------------

#include <math.h>
#include "orbitlib_proc_ConvertKeplerToRectangular.h"

#include "constlib_EarthConstants.h"
static const double GM = earth_const_GM;

typedef double bigflt;


//------------------------------------------------------------------------------
// Kepler's Equation
//------------------------------------------------------------------------------

static bigflt _KeplersEquation          // returns eccentric anomaly
        (
            bigflt meanAnomaly, 
            bigflt eccentricity, 
            bigflt tolerance = 1e-8
        )
{
    bigflt  E = meanAnomaly;
    bigflt  d;

    do  //  kepler's equation
    {
        d = E;
        E = meanAnomaly + eccentricity * sin (E);

    }   while (fabs(d-E)>tolerance);
        
    return E;
}


//page
//------------------------------------------------------------------------------
//
//      Kepler To Satellite Geocentric Inertial Rectangular Coordinates
//
//------------------------------------------------------------------------------

vector<double>  Convert_KeplerToRectangular (const vector<double>& kep) 
{
        const double& A = kep[0];
        const double& e = kep[1];
        const double& i = kep[2];
        const double& N = kep[3];
        const double& w = kep[4];
        const double& m = kep[5];
                                        // eccentric anomaly from kepler's eq.
        bigflt Ecc  = _KeplersEquation (m, e);
        bigflt cosE = cos (Ecc);
        bigflt sinE = sin (Ecc);
                                        // orbital radius
        bigflt radius = A * (1. - e * cosE);

                                        // intermediate variables
        bigflt xp0 = sqrt (1. - e *e); 
        bigflt xp1 = sqrt (A * GM) / radius;
        bigflt xp2 = xp0 * xp1;
        bigflt xp3 = e   * sinE * xp1;

                                        // F = true anomaly
        bigflt F = atan2 (sinE * xp0, cosE - e);

        bigflt cosGF = cos (w + F);
        bigflt sinGF = sin (w + F);

        bigflt cosN  = cos (N);
        bigflt sinN  = sin (N);

        bigflt cosi  = cos (i);
        bigflt sini  = sin (i);

        vector<double> rect(6);
                                        // position
        rect[0] = radius * (cosN * cosGF - sinN * cosi * sinGF);
        rect[1] = radius * (cosN * sinGF * cosi + sinN * cosGF);
        rect[2] = radius * (sini * sinGF);
                                        
                                        // velocity
        rect[3] = xp3 * (cosN * cosGF - sinN * cosi * sinGF)
                - xp2 * (sinN * cosGF * cosi + cosN * sinGF);

        rect[4] = xp3 * (cosN * sinGF * cosi + sinN * cosGF)
                - xp2 * (sinN * sinGF - cosN * cosi * cosGF);
        
        rect[5] = xp3 * (sini * sinGF)
                + xp2 * (sini * cosGF);

        return rect;    
}
