//------------------------------------------------------------------------------
// orbitlib_proc_ConvertRectangularToKepler.c
//
//      29 Feb 2000 Version 1.0
//
//      This procedure computes keplerian elements from geocentric 
//      inertial rectangular elements. The implementation follows "Brouwer-
//      Lyaddane Orbit Generator Routine", X-553-70-223, E. A. Galbreath, 
//      June 1970.
//
//      input: 
//
//	V[0] = A	semi major axis	  (meters)
//      V[1] = e	eccentricity
//      V[2] = i	incliniation	  (radians)
//      V[3] = N	Ascending Node    (radians)
//      V[4] = w	Angle of Perigee  (radians)
//      V[5] = m 	Mean Anomaly	  (radians)
//
//      returns:
//
//	V[0] = x  	(meters)
//	V[1] = y 	(meters)
//	V[2] = z	(meters)
//	V[3] = xdot 	(meters/sec)
//	V[4] = ydot 	(meters/sec)
//	V[5] = zdot 	(meters/sec)
//
//------------------------------------------------------------------------------

#include <math.h>
#include <assert.h>

#include "orbitlib_proc_ConvertRectangularToKepler.h"
#include "constlib_MathConstants.h"
#include "constlib_EarthConstants.h"

static const double twoPi = math_const_twoPi;
static const double GM    = earth_const_GM;

typedef double bigflt;


//------------------------------------------------------------------------------
// Principle Positive Angle
//------------------------------------------------------------------------------

static bigflt _PrincipleAngle (bigflt angle)
{
    bigflt  base = fmod  (angle,  twoPi);
    return (base < 0.) ? (base += twoPi) : (base);
}

//-----------------------------------------------------------------------------
// True Anomaly
//-----------------------------------------------------------------------------

static bigflt _TrueAnomaly (bigflt eccentricAnomaly, bigflt eccentricity)
{
    bigflt &E = eccentricAnomaly;
    bigflt &e = eccentricity;

    bigflt f = 2.*atan( sqrt((1+e)/(1-e)) * tan(E/2) );
    if (f<0.) f += twoPi;
    return f;
}


//page
//------------------------------------------------------------------------
//  

vector<double>  Convert_RectangularToKepler (const vector<double>& V) 
{
    assert(V.size() >= 6);
    const bigflt &xpos = V[0];             // make code more readable
    const bigflt &ypos = V[1];
    const bigflt &zpos = V[2];

    const bigflt &xdot = V[3];
    const bigflt &ydot = V[4];
    const bigflt &zdot = V[5];


//page
//------------------------------------------------------------------------
// conversion code
                                        // radius squared       eq. 4.116
    bigflt radSq = xpos*xpos + ypos*ypos + zpos*zpos;
    bigflt radius= sqrt (radSq);
                                        // velocity squared     eq. 4.117                               
    bigflt velSq = xdot*xdot + ydot*ydot + zdot*zdot;

                                        // R dot V
    bigflt rDotV = xpos*xdot + ypos*ydot + zpos*zdot;

                                        // h = R X V   (h is angular momentum)
    bigflt hz = xpos * ydot - ypos * xdot;
    bigflt hx = ypos * zdot - zpos * ydot;
    bigflt hy = zpos * xdot - xpos * zdot;
  
    bigflt hSq= hz*hz + hy*hy + hx*hx;  // h **2                eq. 4.118
    bigflt h  = sqrt (hSq);             // h
    bigflt P  = hSq/GM;                 // semi latus rectum    eq. 4.118
    
    double A = 1.L / (2.L/radius - velSq/GM);      // semi major axis
    assert(A>0.);

    double e = sqrt (1.L - P/A);	           // eccentricity
    double i = acos (hz/h );                       // inclination

    double N = atan2 (hx,-hy);                     // ascending node
    if (N < 0.) N += twoPi;

    bigflt cosN = cos (N);          	// cos (asc.Node)
    bigflt sinN = sin (N);          	// sin (asc.Node)
    bigflt sini = sin (i);          	// sin (inclination)

    bigflt sinWF;                       // sin (w+f)
    bigflt cosWF;                       // cos (w+f)

    if (i != 0.) {                  	//              eq 2.4-6
        sinWF =  zpos / (radius * sini);       
        cosWF = (xpos * cosN + ypos * sinN) / radius; 

    }else {                             // special case: inc = 0
        sinWF = (ypos * cosN - xpos * sinN) / radius;
        cosWF = (xpos * cosN + ypos * sinN) / radius;
    }
                                                // w + f
    bigflt WF = _PrincipleAngle ( atan2(sinWF,cosWF) );

                                                //            eq 4.106
    bigflt eSinE = rDotV / sqrt (GM * A);   	// e sin (ecc anomaly)
    bigflt eCosE = 1. - radius / A;    	     	// e cos (ecc anomaly)

                                                // eccentric anomaly
    bigflt Ecc = _PrincipleAngle ( atan2(eSinE,eCosE) );

    bigflt F   = _TrueAnomaly (Ecc, e);      	// true anomaly

    vector<double> kep(6);

    kep[0] = A; 				// semi major axis
    kep[1] = e;	             			// eccentricity
    kep[2] = i;                      		// inclination
    kep[3] = N;					// ascending node
    kep[4] = _PrincipleAngle (WF - F);          // argument of perigee
    kep[5] = _PrincipleAngle (Ecc - eSinE);     // mean anomaly

    return kep;
}
