//==============================================================================
// GreenwichHourAngle - Apparent Greenwich Hour Angle of Equinox
//
// 05 Oct 1999  Version 1.0
//
// This model adapted from "Exact Closed-Form Geolocation Algorithm for Earth
// Survey Sensors", F.S. Patt, W. W. Gregg, Int. Journal of Remote Sensing, 1993.
// The model follows FORTRAN procedures (gha2000) as coded by F.S. Patt 
// for the NASA SeaWifs Project.
// 
// Also, see "The Astronomical Almanac for the Year 1998", section B, and
// "Astronomical Algorthims", Jean Meeus.
//
// This procedure takes a long double Julian Day and fraction, and returns the
// hour angle in radians.
//
//==============================================================================


#include <math.h>
#include "constlib_MathConstants.h"
#include "orbitlib_proc_GreenwichHourAngle.h"

typedef double bigflt;


static double PositivePrincipleAngle (double angle)
{
    double  base = fmod  (angle,  math_const_twoPi);
    return (base < 0.) ? (base += math_const_twoPi) : (base);
}

//page
//------------------------------------------------------------------------------
bigflt GreenwichHourAngle (bigflt JD)
{
						// make code more readable
    const  bigflt &rToD  = math_const_radiansToDegrees;    

    const  bigflt solarDay = 1.0027379093;      // sideral days

    bigflt jd2000 = JD - 2451545.5L ;           // days since year 2000

    bigflt IJD  = (long)(JD + 0.5L) -0.5L;      // integral julian day
    bigflt fday = JD - IJD;                     // fraction Julian day

    bigflt Tu = (IJD - 2451545.0L) / 36525L;    // Julian centuries

    bigflt gmst   = 24110.54841L                // mean gha (sec) at 0 UT.
                  + 8640184.812866L * Tu        // Ast. Almanac,1998,B6
                  + 0.093104L       * Tu * Tu
                  - 6.2e-6L         * Tu * Tu * Tu;
           
           gmst   = gmst/86400.0L *360.0L;      // gha degrees

                                        // Sun Mean Longitude (deg)
    bigflt xls    = fmod (280.46592L + 0.9856473516L * jd2000, 360.0L);

                                        // Sun Mean Anomaly (deg)
    bigflt gs     = fmod (357.52772L + 0.9856002831L * jd2000, 360.0L);

                                        // Moon Mean Longitude (deg)
    bigflt xlm    = fmod (218.31643L + 13.17639648L  * jd2000, 360.0L);

                                        // Ascending Node of Lunar Mean Orbit
                                        // (deg)
    bigflt omega  = fmod (125.04452L - 0.0529537648L * jd2000, 360.0L);

                                        // Nutation in Longitude
                                        // (Delta Psi,  deg)
    bigflt dpsi   = -17.1996L * sin (omega / rToD)
                  +   0.2062L * sin (omega / rToD * 2.0L)
                  -   1.3187L * sin (xls   / rToD * 2.0L)
                  +   0.1426L * sin (gs    / rToD)
                  -   0.2274L * sin (xlm   / rToD * 2.0L);

                                        // Mean Obliquity of the Ecliptic (deg)
    bigflt epsm   = 23.439291L - 3.560e-7L  * jd2000;
   
                                        // Nutation in Obliquity
                                        // (Delta Epsilon, arc sec)
    bigflt deps   = 9.2025L * cos (omega  / rToD) 
                  + 0.5736L * cos (2* xls / rToD);

                                        // True Obliquity of the Ecliptic (deg)
    bigflt eps    = epsm + deps/3600L;
           dpsi   = dpsi/3600L;

    bigflt gha    = gmst + dpsi * cos (eps / rToD) + fday * 360L *solarDay;
    return PositivePrincipleAngle (gha / rToD);
}   

