//==============================================================================
// timelib_class_datetime.h- date and time class
//
// 03 Nov 1999 Version 1.0
//
// Note:
//    1. years are four-digit years. Day-of-year begins at zero.
//
//    2. ConvertDateToString format Codes:
//	 %Y4 - four digit year;
//	 %Y2 - two  digit year  (00-99)
//	 %Mn - two  digit month (1 -12)
//       %Ma - three character month
//	 %MA - complete month name
//	 %Dn - day of month (1-31)
//	 %Wn - day of week  (1 -7)
//       %Wa - three character day of week
//	 %WA - complete day of week name 
//	 %hh - hour
//	 %mm - minute
//	 %ss - integer second
//       %SS - floating second
//
//    3. ConvertDateToString format Codes:
//	 %Y4 - four digit year;
//	 %Mn - two  digit month (1 -12)
//       %Ma - three character month
//	 %MA - complete month name
//	 %Dn - day of month (1-31)
//	 %hh - hour
//	 %mm - minute
//	 %sx - second. "x" is fieldwidth
//       %Sx - same as sx
//
//
// reference 1.  "Astronomical Algorithms", Jean Meeus, Willman-Bell, Inc., 1991
//
// reference 2.  "Almanac for Computers 1983", Nautical Almanac Office, United 
// States Naval Observatory, 1983.
//
//==============================================================================

#if !defined(TIMELIB_DATETIME)
#define      TIMELIB_DATETIME

#include <math.h>
#include <string>
#include <string.h>
#include <stdlib.h>
#include <time.h>
using namespace std;


//page
//-----------------------------------------------------------------------------
// datetime class- date and time conversion
//
//-----------------------------------------------------------------------------

class date_time
{
private:
    long _DayToHourMinuteSecond 
    (double day, size_t& hour, size_t& minute, double& second) const;

    void _GetGregorianYearMonthDay() const;

    void _initial (double time = 0.);	

    size_t _GetNumericMonth (const string& month);	

    double _time;		// time stored as Julian Day

    double _itime;		// time for following calender date:
    double _day,  _sec;		//
    size_t _year, _month, _hour, _min;


//page
//------------------------------------------------------------------------------
// Constructors
//
//------------------------------------------------------------------------------

public:
    date_time ();
    date_time (const date_time& datetime );
//  date_time ( time(0) );			// current time
    explicit date_time (time_t sec1970  );
    explicit date_time (double julianDay);
 

//------------------------------------------------------------------------------
// Arithemetic and Comparison Operattors
//
//------------------------------------------------------------------------------

public:
    date_time  operator+= (const double julianDays);
    date_time  operator-= (const double julianDays);
    double    operator+  (double julianDays) const;
    double    operator-  (const date_time& datetime) const;
    double    operator-  (double julianDays) const;
 
    bool operator== (const date_time& datetime) const;
    bool operator!= (const date_time& datetime) const; 
    bool operator<= (const date_time& datetime) const;
    bool operator>= (const date_time& datetime) const;
    bool operator>  (const date_time& datetime) const;
    bool operator<  (const date_time& datetime) const;


//page
//------------------------------------------------------------------------------
// Set Time Operations
//
//------------------------------------------------------------------------------

public:
    date_time SetJulianDay (double julianDate);

    date_time SetGregorianDate (
        size_t year, size_t month,  size_t day, 
	size_t hour, size_t minute, double second
    );

    date_time SetGregorianDate (
        size_t year, string month,  size_t day, 
	size_t hour, size_t minute, double second
    );


    date_time SetGregorianYearMonthDay (
	size_t year, size_t month, double dayOfMonth
    );

    date_time SetGregorianYearMonthDay (
	size_t year, string month, double dayOfMonth
    );

    date_time SetGregorianYearDaySecond (
	size_t year, size_t day0OfYear, double second
    );


//------------------------------------------------------------------------------
// Get Time Operations
//
//------------------------------------------------------------------------------

public:
    double GetJulianDay() const;

    void   GetGregorianYearMonthDay (
	   size_t& year, size_t& month, double& day
    ) const;


    void   GetGregorianDate ( 
           size_t& year, size_t& month,  size_t& day, 
	   size_t& hour, size_t& minute, double& second
    ) const;


    void   GetGregorianYearDaySecond (
	   size_t& year, size_t& day0OfYear, double& second
    ) const;


//page
//------------------------------------------------------------------------------
// Other Operations
//
//------------------------------------------------------------------------------

public:
    size_t  GetNumericDayofWeek    (size_t* day = 0) const;
    double  GetJulianDateAt0hr     (double* date= 0) const;

    string ConvertDateToString (
           const string format = "%Y4/%Mn/%Dn %hh:%mm:%ss"
    ) const;

    date_time&
    ConvertStringToDate( const string& dateStr, const string& formatStr );

    bool operator!() {return _time <= 0.;}
};
#endif
